# -*- coding: utf-8 -*-
import pytest
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__spanners__Beam_01():

    target = Container(scoretools.make_notes([0] * 4, [(1, 4)]))
    beam = Beam()
    attach(beam, target[0:3])
    beam = Beam()
    attach(beam, target[3:])

    assert format(target) == stringtools.normalize(
        r'''
        {
            c'4 [
            c'4
            c'4 ]
            c'4 [ ]
        }
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__spanners__Beam_02():
    r'''With start and stop reversed.
    '''

    target = Container(scoretools.make_notes([0] * 4, [(1, 4)]))
    beam = Beam()
    attach(beam, target[0:3])
    beam = Beam()
    attach(beam, target[3:])

    assert format(target) == stringtools.normalize(
        r'''
        {
            c'4 [
            c'4
            c'4 ]
            c'4 [ ]
        }
        '''
        )

    string = r'''\relative c' { c [ c c ] c ] [ }'''
    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__spanners__Beam_03():

    target = Container(scoretools.make_notes([0] * 4, [(1, 4)]))
    beam = Beam()
    attach(beam, target[:])
    beam = Beam()
    attach(beam, target[1:3])

    assert format(target) == stringtools.normalize(
        r'''
        {
            c'4 [
            c'4 [
            c'4 ]
            c'4 ]
        }
        '''
        )

    assert pytest.raises(Exception, "LilyPondParser()(format(target))")


def test_lilypondparsertools_LilyPondParser__spanners__Beam_04():

    target = Container(scoretools.make_notes([0] * 4, [(1, 4)]))
    beam = Beam()
    attach(beam, target[:3])
    beam = Beam()
    attach(beam, target[2:])

    assert format(target) == stringtools.normalize(
        r'''
        {
            c'4 [
            c'4
            c'4 ] [
            c'4 ]
        }
        '''
        )

    assert pytest.raises(Exception, "LilyPondParser()(format(target))")


def test_lilypondparsertools_LilyPondParser__spanners__Beam_05():

    string = "{ c'4 [ c'4 c'4 c'4 }"
    assert pytest.raises(Exception, "LilyPondParser()(string)")


def test_lilypondparsertools_LilyPondParser__spanners__Beam_06():

    string = "{ c'4 c'4 c'4 c'4 ] }"
    result = LilyPondParser()(string)
    assert not inspect_(result[-1]).get_spanners()


def test_lilypondparsertools_LilyPondParser__spanners__Beam_07():
    r'''With direction.
    '''

    target = Container(scoretools.make_notes([0] * 4, [(1, 4)]))
    beam = Beam(direction=Up)
    attach(beam, target[0:3])
    beam = Beam(direction=Down)
    attach(beam, target[3:])

    assert format(target) == stringtools.normalize(
        r'''
        {
            c'4 ^ [
            c'4
            c'4 ]
            c'4 _ [ ]
        }
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result



# -*- coding: utf-8 -*-
from abjad.tools.topleveltools import iterate


def fill_measures_with_minimal_number_of_notes(
    argument,
    decrease_durations_monotonically=True,
    iterctrl=None,
    ):
    '''Fills measures in `argument` with minimal number of notes that decrease
    durations monotonically:

    ::

        >>> measure = Measure((5, 18), [], implicit_scaling=True)

    ::

        >>> scoretools.fill_measures_with_minimal_number_of_notes(
        ...     measure, decrease_durations_monotonically=True)

    ..  doctest::

        >>> print(format(measure))
        {
            \time 5/18
            \scaleDurations #'(8 . 9) {
                c'4 ~
                c'16
            }
        }

    Fill measures in `argument` with minimal number of notes that increase
    durations monotonically:

    ::

        >>> measure = Measure((5, 18), [])
        >>> measure.implicit_scaling = True

    ::

        >>> scoretools.fill_measures_with_minimal_number_of_notes(
        ...     measure, decrease_durations_monotonically=False)

    ..  doctest::

        >>> print(format(measure))
        {
            \time 5/18
            \scaleDurations #'(8 . 9) {
                c'16 ~
                c'4
            }
        }

    Returns none.
    '''
    from abjad.tools import scoretools

    if iterctrl is None:
        iterctrl = lambda measure, i: True
    for i, measure in enumerate(iterate(argument).by_class(scoretools.Measure)):
        if iterctrl(measure, i):
            time_signature = measure.time_signature
            written_duration = time_signature.duration / time_signature.implied_prolation
            notes = scoretools.make_notes(
                0,
                written_duration,
                decrease_durations_monotonically=decrease_durations_monotonically)
            measure[:] = notes



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_MutationAgent_extract_01():
    r'''Extracts note.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:])
    glissando = spannertools.Glissando()
    attach(glissando, voice[:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [ \glissando
            d'8 \glissando
            e'8 \glissando
            f'8 ]
        }
        '''
        )

    note = voice[1]
    mutate(note).extract()

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [ \glissando
            e'8 \glissando
            f'8 ]
        }
        '''
        )

    assert inspect_(note).is_well_formed()
    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_extract_02():
    r'''Extracts multiple notes.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:])
    glissando = spannertools.Glissando()
    attach(glissando, voice[:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [ \glissando
            d'8 \glissando
            e'8 \glissando
            f'8 ]
        }
        '''
        )

    notes = voice[:2]
    for note in notes:
        mutate(note).extract()

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            e'8 [ \glissando
            f'8 ]
        }
        '''
        )

    for note in notes:
        assert inspect_(note).is_well_formed()

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_extract_03():
    r'''Extracts container.
    '''

    staff = Staff()
    staff.append(Container("c'8 d'8"))
    staff.append(Container("e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    container = staff[0]
    mutate(container).extract()

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'8
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    assert not container
    assert inspect_(staff).is_well_formed()


def test_agenttools_MutationAgent_extract_04():
    r'''Extracts multiple containers.
    '''

    voice = Voice()
    voice.append(Container("c'8 d'8"))
    voice.append(Container("e'8 f'8"))
    voice.append(Container("g'8 a'8"))
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    glissando = spannertools.Glissando()
    attach(glissando, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ \glissando
                d'8 \glissando
            }
            {
                e'8 \glissando
                f'8 \glissando
            }
            {
                g'8 \glissando
                a'8 ]
            }
        }
        '''
        )

    containers = voice[:2]
    for container in containers:
        mutate(container).extract()

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [ \glissando
            d'8 \glissando
            e'8 \glissando
            f'8 \glissando
            {
                g'8 \glissando
                a'8 ]
            }
        }
        '''
        )

    for container in containers:
        assert not container

    assert inspect_(voice).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_IterationAgent_by_class_01():

    staff = Staff("<e' g' c''>8 a'8 r8 <d' f' b'>8 r2")
    chords = iterate(staff).by_class(Chord)
    chords = list(chords)

    assert chords[0] is staff[0]
    assert chords[1] is staff[3]


def test_agenttools_IterationAgent_by_class_02():

    staff = Staff("<e' g' c''>8 a'8 r8 <d' f' b'>8 r2")
    chords = iterate(staff).by_class(Chord, reverse=True)
    chords = list(chords)

    assert chords[0] is staff[3]
    assert chords[1] is staff[0]


def test_agenttools_IterationAgent_by_class_03():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = list(iterate(staff).by_leaf())
    beam = Beam()
    attach(beam, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    components = list(iterate(beam).by_class(reverse=True))
    leaves = list(iterate(staff).by_leaf())

    assert components[0] is leaves[-1]
    assert components[1] is leaves[-2]
    assert components[2] is leaves[-3]
    assert components[3] is leaves[-4]


def test_agenttools_IterationAgent_by_class_04():

    staff = Staff("c'8 d'8 e'8 f'8")
    spanner = Beam()
    attach(spanner, staff[2:])
    components = iterate(spanner).by_class()
    components = list(components)

    assert components == staff[2:]


def test_agenttools_IterationAgent_by_class_05():

    staff = Staff([Voice("c'8 d'8"), Voice("e'8 f'8 g'8")])
    tuplet = Tuplet(Multiplier(2, 3), staff[1][:])
    staff.is_simultaneous = True
    containers = iterate(staff).by_class(Container, reverse=True)
    containers = list(containers)

    assert containers[0] is staff
    assert containers[1] is staff[1]
    assert containers[2] is tuplet
    assert containers[3] is staff[0]


def test_agenttools_IterationAgent_by_class_06():

    staff = Staff([Voice("c'8 d'8"), Voice("e'8 f'8 g'8")])
    tuplet = Tuplet(Multiplier(2, 3), staff[1][:])
    staff.is_simultaneous = True
    containers = iterate(staff).by_class(Container)
    containers = list(containers)

    assert containers[0] is staff
    assert containers[1] is staff[0]
    assert containers[2] is staff[1]
    assert containers[3] is tuplet


def test_agenttools_IterationAgent_by_class_07():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
        }
        ''',
        )

    leaves = iterate(staff).by_leaf(reverse=True)
    leaves = list(leaves)

    assert leaves[0] is staff[2][1]
    assert leaves[1] is staff[2][0]
    assert leaves[2] is staff[1][1]
    assert leaves[3] is staff[1][0]
    assert leaves[4] is staff[0][1]
    assert leaves[5] is staff[0][0]


def test_agenttools_IterationAgent_by_class_08():
    r'''Optional start and stop keyword parameters.
    '''

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")
    leaves = iterate(staff).by_leaf(start=3, reverse=True)
    leaves = list(leaves)

    assert leaves[0] is staff[1][0]
    assert leaves[1] is staff[0][1]
    assert leaves[2] is staff[0][0]
    assert len(leaves) == 3

    leaves = iterate(staff).by_leaf(start=0, stop=3, reverse=True)
    leaves = list(leaves)

    assert leaves[0] is staff[2][1]
    assert leaves[1] is staff[2][0]
    assert leaves[2] is staff[1][1]
    assert len(leaves) == 3

    leaves = iterate(staff).by_leaf(start=2, stop=4, reverse=True)
    leaves = list(leaves)

    assert leaves[0] is staff[1][1]
    assert leaves[1] is staff[1][0]
    assert len(leaves) == 2


def test_agenttools_IterationAgent_by_class_09():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
        }
        ''',
        )

    leaves = iterate(staff).by_leaf()
    leaves = list(leaves)

    assert leaves[0] is staff[0][0]
    assert leaves[1] is staff[0][1]
    assert leaves[2] is staff[1][0]
    assert leaves[3] is staff[1][1]
    assert leaves[4] is staff[2][0]
    assert leaves[5] is staff[2][1]


def test_agenttools_IterationAgent_by_class_10():
    r'''Optional start and stop keyword parameters.
    '''

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")
    leaves = iterate(staff).by_leaf(start=3)
    leaves = list(leaves)

    assert leaves[0] is staff[1][1]
    assert leaves[1] is staff[2][0]
    assert leaves[2] is staff[2][1]
    assert len(leaves) == 3

    leaves = iterate(staff).by_leaf(start=0, stop=3)
    leaves = list(leaves)

    assert leaves[0] is staff[0][0]
    assert leaves[1] is staff[0][1]
    assert leaves[2] is staff[1][0]
    assert len(leaves) == 3

    leaves = iterate(staff).by_leaf(start=2, stop=4)
    leaves = list(leaves)

    assert leaves[0] is staff[1][0]
    assert leaves[1] is staff[1][1]
    assert len(leaves) == 2


def test_agenttools_IterationAgent_by_class_11():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
        }
        ''',
        )

    measures = iterate(staff).by_class(Measure, reverse=True)
    measures = list(measures)

    assert measures[0] is staff[2]
    assert measures[1] is staff[1]
    assert measures[2] is staff[0]


def test_agenttools_IterationAgent_by_class_12():
    r'''Optional start and stop keyword paramters.
    '''

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")
    measures = iterate(staff).by_class(Measure, start=1, reverse=True)
    measures = list(measures)

    assert measures[0] is staff[1]
    assert measures[1] is staff[0]
    assert len(measures) == 2

    measures = iterate(staff).by_class(Measure, stop=2, reverse=True)
    measures = list(measures)

    assert measures[0] is staff[2]
    assert measures[1] is staff[1]
    assert len(measures) == 2


def test_agenttools_IterationAgent_by_class_13():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
        }
        ''',
        )

    measures = iterate(staff).by_class(Measure)
    measures = list(measures)

    assert measures[0] is staff[0]
    assert measures[1] is staff[1]
    assert measures[2] is staff[2]


def test_agenttools_IterationAgent_by_class_14():
    r'''Optional start and stop keyword paramters.
    '''

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")
    measures = iterate(staff).by_class(Measure, start=1)
    measures = list(measures)

    assert measures[0] is staff[1]
    assert measures[1] is staff[2]
    assert len(measures) == 2

    measures = iterate(staff).by_class(Measure, stop=2)
    measures = list(measures)

    assert measures[0] is staff[0]
    assert measures[1] is staff[1]
    assert len(measures) == 2


def test_agenttools_IterationAgent_by_class_15():

    staff = Staff("<e' g' c''>8 a'8 r8 <d' f' b'>8 r2")
    notes_and_chords = iterate(staff).by_class((Note, Chord), reverse=True)
    notes_and_chords = list(notes_and_chords)

    assert len(notes_and_chords) == 3
    assert notes_and_chords[0] is staff[3]
    assert notes_and_chords[1] is staff[1]
    assert notes_and_chords[2] is staff[0]


def test_agenttools_IterationAgent_by_class_16():

    staff = Staff("<e' g' c''>8 a'8 r8 <d' f' b'>8 r2")
    notes_and_chords = iterate(staff).by_class((Note, Chord))
    notes_and_chords = list(notes_and_chords)

    assert len(notes_and_chords) == 3
    assert notes_and_chords[0] is staff[0]
    assert notes_and_chords[1] is staff[1]
    assert notes_and_chords[2] is staff[3]


def test_agenttools_IterationAgent_by_class_17():

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    staff.append(Measure((2, 8), "g'8 a'8"))

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
        }
        '''
        )

    notes = iterate(staff).by_class(Note, reverse=True)
    notes = list(notes)

    assert notes[0] is staff[2][1]
    assert notes[1] is staff[2][0]
    assert notes[2] is staff[1][1]
    assert notes[3] is staff[1][0]
    assert notes[4] is staff[0][1]
    assert notes[5] is staff[0][0]


def test_agenttools_IterationAgent_by_class_18():
    r'''Optional start and stop keyword parameters.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    staff.append(Measure((2, 8), "g'8 a'8"))

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
        }
        '''
        )

    notes = iterate(staff).by_class(Note, reverse=True, start=3)
    notes = list(notes)

    assert notes[0] is staff[1][0]
    assert notes[1] is staff[0][1]
    assert notes[2] is staff[0][0]
    assert len(notes) == 3

    notes = iterate(staff).by_class(Note, reverse=True, start=0, stop=3)
    notes = list(notes)

    assert notes[0] is staff[2][1]
    assert notes[1] is staff[2][0]
    assert notes[2] is staff[1][1]
    assert len(notes) == 3

    notes = iterate(staff).by_class(Note, reverse=True, start=2, stop=4)
    notes = list(notes)

    assert notes[0] is staff[1][1]
    assert notes[1] is staff[1][0]
    assert len(notes) == 2


def test_agenttools_IterationAgent_by_class_19():

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    staff.append(Measure((2, 8), "g'8 a'8"))

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
        }
        '''
        )

    notes = iterate(staff).by_class(Note)
    notes = list(notes)

    assert notes[0] is staff[0][0]
    assert notes[1] is staff[0][1]
    assert notes[2] is staff[1][0]
    assert notes[3] is staff[1][1]
    assert notes[4] is staff[2][0]
    assert notes[5] is staff[2][1]


def test_agenttools_IterationAgent_by_class_20():
    r'''Optional start and stop keyword parameters.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    staff.append(Measure((2, 8), "g'8 a'8"))

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
        }
        '''
        )

    notes = iterate(staff).by_class(Note, start=3)
    notes = list(notes)

    assert notes[0] is staff[1][1]
    assert notes[1] is staff[2][0]
    assert notes[2] is staff[2][1]
    assert len(notes) == 3

    notes = iterate(staff).by_class(Note, start=0, stop=3)
    notes = list(notes)

    assert notes[0] is staff[0][0]
    assert notes[1] is staff[0][1]
    assert notes[2] is staff[1][0]
    assert len(notes) == 3

    notes = iterate(staff).by_class(Note, start=2, stop=4)
    notes = list(notes)

    assert notes[0] is staff[1][0]
    assert notes[1] is staff[1][1]
    assert len(notes) == 2


def test_agenttools_IterationAgent_by_class_21():

    staff = Staff("<e' g' c''>8 a'8 r8 <d' f' b'>8 r2")
    rests = iterate(staff).by_class(Rest, reverse=True)
    rests = list(rests)

    assert rests[0] is staff[4]
    assert rests[1] is staff[2]


def test_agenttools_IterationAgent_by_class_22():

    staff = Staff("<e' g' c''>8 a'8 r8 <d' f' b'>8 r2")
    rests = iterate(staff).by_class(Rest)
    rests = list(rests)

    assert rests[0] is staff[2]
    assert rests[1] is staff[4]


def test_agenttools_IterationAgent_by_class_23():

    score_1 = Score([Staff("c'8 d'8 e'8 f'8")])
    score_2 = Score([Staff("c'1"), Staff("g'1")])
    scores = [score_1, score_2]
    scores = iterate(scores).by_class(Score, reverse=True)
    scores = list(scores)

    assert scores[0] is score_2
    assert scores[1] is score_1


def test_agenttools_IterationAgent_by_class_24():

    score_1 = Score([Staff("c'8 d'8 e'8 f'8")])
    score_2 = Score([Staff("c'1"), Staff("g'1")])
    scores = [score_1, score_2]
    scores = iterate(scores).by_class(Score)
    scores = list(scores)

    assert scores[0] is score_1
    assert scores[1] is score_2


def test_agenttools_IterationAgent_by_class_25():

    staff = Staff("<e' g' c''>8 a'8 s8 <d' f' b'>8 s2")
    skips = iterate(staff).by_class(scoretools.Skip)
    skips = list(skips)

    assert skips[0] is staff[2]
    assert skips[1] is staff[4]


def test_agenttools_IterationAgent_by_class_26():

    staff = Staff("<e' g' c''>8 a'8 s8 <d' f' b'>8 s2")
    skips = iterate(staff).by_class(scoretools.Skip, reverse=True)
    skips = list(skips)

    assert skips[0] is staff[4]
    assert skips[1] is staff[2]


def test_agenttools_IterationAgent_by_class_27():

    score = Score(4 * Staff([]))
    score[0].name = '1'
    score[1].name = '2'
    score[2].name = '3'
    score[3].name = '4'
    staves = iterate(score).by_class(Staff, reverse=True)

    for i, staff in enumerate(staves):
        assert staff.name == str(4 - i)


def test_agenttools_IterationAgent_by_class_28():

    score = Score(4 * Staff([]))
    score[0].name = '1'
    score[1].name = '2'
    score[2].name = '3'
    score[3].name = '4'
    staves = iterate(score).by_class(Staff)

    for i, staff in enumerate(staves):
        assert staff.name == str(i + 1)


def test_agenttools_IterationAgent_by_class_29():

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    tuplet_0 = Tuplet(Multiplier(2, 3), staff[:3])
    tuplet_1 = Tuplet(Multiplier(2, 3), staff[-3:])
    tuplets = iterate(staff).by_class(Tuplet, reverse=True)
    tuplets = list(tuplets)

    assert tuplets[0] is tuplet_1
    assert tuplets[1] is tuplet_0


def test_agenttools_IterationAgent_by_class_30():

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    tuplet_0 = Tuplet(Multiplier(2, 3), staff[:3])
    tuplet_1 = Tuplet(Multiplier(2, 3), staff[-3:])
    tuplets = iterate(staff).by_class(Tuplet)
    tuplets = list(tuplets)

    assert tuplets[0] is tuplet_0
    assert tuplets[1] is tuplet_1


def test_agenttools_IterationAgent_by_class_31():

    voice_1 = Voice("c'8 d'8 e'8 f'8")
    voice_2 = Voice("c'4 b4")
    staff = Staff([voice_1, voice_2])
    staff.is_simultaneous = True
    voices = iterate(staff).by_class(Voice, reverse=True)
    voices = list(voices)

    assert voices[0] is voice_2
    assert voices[1] is voice_1


def test_agenttools_IterationAgent_by_class_32():

    voice_1 = Voice("c'8 d'8 e'8 f'8")
    voice_2 = Voice("c'4 b4")
    staff = Staff([voice_1, voice_2])
    staff.is_simultaneous = True
    voices = iterate(staff).by_class(Voice)
    voices = list(voices)

    assert voices[0] is voice_1
    assert voices[1] is voice_2



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedPitch___init___01():
    r'''Initialize with number.
    '''

    assert isinstance(pitchtools.NumberedPitch(0), pitchtools.NumberedPitch)
    assert isinstance(pitchtools.NumberedPitch(0.5), pitchtools.NumberedPitch)
    assert isinstance(pitchtools.NumberedPitch(12), pitchtools.NumberedPitch)
    assert isinstance(pitchtools.NumberedPitch(12.5), pitchtools.NumberedPitch)
    assert isinstance(pitchtools.NumberedPitch(-12), pitchtools.NumberedPitch)
    assert isinstance(pitchtools.NumberedPitch(-12.5), pitchtools.NumberedPitch)


def test_pitchtools_NumberedPitch___init___02():
    r'''Initialize with other numbered pitch instance.
    '''

    numbered_pitch_1 = pitchtools.NumberedPitch(13)
    numbered_pitch_2 = pitchtools.NumberedPitch(numbered_pitch_1)

    assert isinstance(numbered_pitch_1, pitchtools.NumberedPitch)
    assert isinstance(numbered_pitch_2, pitchtools.NumberedPitch)



# -*- coding: utf-8 -*-
from abjad.tools import schemetools
from abjad.tools.spannertools.Spanner import Spanner


class StaffLinesSpanner(Spanner):
    r'''Staff lines spanner.

    ..  container:: example

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.StaffLinesSpanner(lines=1)
            >>> attach(spanner, staff[1:3])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'8
                \stopStaff
                \once \override Staff.StaffSymbol.line-count = 1
                \startStaff
                d'8
                e'8
                \stopStaff
                \startStaff
                f'8
            }

    Stops and restarts staff on first leaf in spanner.

    Overrides ``line-count`` attribute of LilyPond ``Staff.StaffSymbol`` grob
    on first leaf in spanner.

    Stops and restarts staff on last leaf in spanner.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_lines',
        '_forbid_restarting',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        lines=5,
        forbid_restarting=None,
        overrides=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        if isinstance(lines, int) and 0 < lines:
            self._lines = lines
        elif (isinstance(lines, (tuple, list)) and
            all(isinstance(x, (int, float)) for x in lines)):
            self._lines = tuple(lines)
        else:
            message = 'must be integer or a sequence of integers: {!r}.'
            message = message.format(lines)
            raise ValueError(message)
        self._lines = lines
        if forbid_restarting is not None:
            forbid_restarting = bool(forbid_restarting)
        self._forbid_restarting = forbid_restarting

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new._lines = self.lines

    def _format_after_leaf(self, leaf):
        result = []
        if self._is_my_last_leaf(leaf) and not self.forbid_restarting:
            result.append(r'\stopStaff')
            result.append(r'\startStaff')
        return result

    def _format_before_leaf(self, leaf):
        from abjad.tools import lilypondnametools
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(r'\stopStaff')
            if isinstance(self.lines, int):
                override = lilypondnametools.LilyPondGrobOverride(
                    context_name='Staff',
                    grob_name='StaffSymbol',
                    is_once=True,
                    property_path='line-count',
                    value=self.lines,
                    )
                string = override.override_string
                result.append(string)
            else:
                override = lilypondnametools.LilyPondGrobOverride(
                    context_name='Staff',
                    grob_name='StaffSymbol',
                    is_once=True,
                    property_path='line-positions',
                    value=schemetools.SchemeVector(self.lines),
                    )
                string = override.override_string
                result.append(string)
            result.append(r'\startStaff')
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def forbid_restarting(self):
        r'''Is true if staff lines spanner is forbidden from re-stopping and
        re-starting the staff on its last leaf. Otherwise false.

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.StaffLinesSpanner(
            ...     lines=1,
            ...     forbid_restarting=True,
            ...     )
            >>> attach(spanner, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \stopStaff
                \once \override Staff.StaffSymbol.line-count = 1
                \startStaff
                c'8
                d'8
                e'8
                f'8
            }

        This is useful when the final leaf of a score is covered by a staff
        lines spanner, to prevent unexpected LilyPond typesetting behavior.

        Returns true or false.
        '''
        return self._forbid_restarting

    @property
    def lines(self):
        r'''Gets line of staff lines spanner.

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.StaffLinesSpanner(lines=1)
            >>> attach(spanner, staff[1:3])
            >>> show(staff) # doctest: +SKIP

        ::

            >>> spanner.lines
            1

        Returns nonnegative integer.
        '''
        return self._lines



# -*- coding: utf-8 -*-
from __future__ import print_function
import datetime
import os
import sys
from abjad.tools import stringtools
from abjad.tools.commandlinetools.ScorePackageScript import ScorePackageScript
try:
    import pathlib
except ImportError:
    import pathlib2 as pathlib


class ManageScoreScript(ScorePackageScript):
    '''
    Manages score package.

    ..  shell::

        ajv score --help

    '''

    ### CLASS VARIABLES ###

    alias = 'score'
    short_description = 'Manage score packages.'

    ### PRIVATE METHODS ###

    def _handle_create(
        self,
        title,
        year,
        composer_email=None,
        composer_github=None,
        composer_library=None,
        composer_name=None,
        composer_website=None,
        force=False,
        ):
        print('Creating score package {!r}...'.format(title))
        score_package_name = stringtools.to_accent_free_snake_case(title)
        outer_target_path = pathlib.Path(score_package_name).absolute()
        inner_target_path = outer_target_path.joinpath(score_package_name)
        if outer_target_path.exists() and not force:
            message = '    Directory {!s} already exists.'
            message = message.format(score_package_name)
            print(message)
            sys.exit(1)
        metadata = {
            'score_package_name': score_package_name,
            'composer_email': composer_email,
            'composer_full_name': composer_name,
            'composer_github_username': composer_github,
            'composer_library_package_name': composer_library,
            'score_title': title,
            'year': year,
            }
        boilerplate_path = self._get_boilerplate_path()
        outer_source_path = boilerplate_path.joinpath('example_score')
        inner_source_path = outer_source_path.joinpath('example_score')
        suffixes = ('.py', '.tex', '.ly', '.ily', '.md', '.yml')
        for path in self._copy_tree(
            outer_source_path,
            outer_target_path,
            recurse=False,
            ):
            if path.is_file() and path.suffix in suffixes:
                self._template_file(path, **metadata)
        for path in self._copy_tree(
            inner_source_path,
            inner_target_path,
            recurse=True,
            ):
            if path.is_file() and path.suffix in suffixes:
                self._template_file(path, **metadata)
        self._setup_paths(inner_target_path)
        self._write_json(
            dict(
                composer_email=composer_email,
                composer_github=composer_github,
                composer_library=composer_library,
                composer_name=composer_name,
                composer_website=composer_website,
                title=title,
                year=int(year),
                ),
            inner_target_path.joinpath('metadata.json'),
            )
        print('    Created {path!s}{sep}'.format(
            path=score_package_name,
            sep=os.path.sep))

    def _process_args(self, arguments):
        if arguments.new:
            self._handle_create(
                title=arguments.new,
                year=int(arguments.year),
                composer_email=arguments.composer_email,
                composer_github=arguments.composer_github,
                composer_library=arguments.composer_library,
                composer_name=arguments.composer_name,
                composer_website=arguments.composer_website,
                force=arguments.force,
                )

    def _setup_argument_parser(self, parser):
        action_group = parser.add_argument_group('actions')
        action_group = action_group.add_mutually_exclusive_group(required=True)
        action_group.add_argument(
            '--new', '-N',
            help='create a new score',
            metavar='TITLE',
            )
        create_group = parser.add_argument_group('--new options')
        create_group.add_argument(
            '--year', '-y',
            default=str(datetime.datetime.today().year),
            metavar='YEAR',
            )
        create_group.add_argument(
            '--composer-name', '-n',
            default='A Composer',
            metavar='NAME',
            )
        create_group.add_argument(
            '--composer-email', '-e',
            default='composer@email.com',
            metavar='EMAIL',
            )
        create_group.add_argument(
            '--composer-github', '-g',
            default='composer',
            metavar='GITHUB_USERNAME',
            )
        create_group.add_argument(
            '--composer-library', '-l',
            default='library',
            metavar='LIBRARY_NAME',
            )
        create_group.add_argument(
            '--composer-website', '-w',
            default='www.composer.com',
            metavar='WEBSITE',
            )
        common_group = parser.add_argument_group('common options')
        common_group.add_argument(
            '--force', '-f',
            action='store_true',
            help='force overwriting',
            )



# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class LilyPondFormatManager(AbjadObject):
    r'''Manages LilyPond formatting logic.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'LilyPond formatting'

    lilypond_color_constants = (
        'black',
        'blue',
        'center',
        'cyan',
        'darkblue',
        'darkcyan',
        'darkgreen',
        'darkmagenta',
        'darkred',
        'darkyellow',
        'down',
        'green',
        'grey',
        'left',
        'magenta',
        'red',
        'right',
        'up',
        'white',
        'yellow',
        )

    indent = '    '

    ### PRIVATE METHODS ###

    @staticmethod
    def _collect_indicators(component):
        from abjad.tools import markuptools
        from abjad.tools.topleveltools import inspect_
        expressions = []
        for parent in inspect_(component).get_parentage(include_self=True):
            result = inspect_(parent).get_indicators(unwrap=False)
            expressions.extend(result)
            result = parent._get_spanner_indicators(unwrap=False)
            expressions.extend(result)
        up_markup = []
        down_markup = []
        neutral_markup = []
        scoped_expressions = []
        nonscoped_expressions = []
        # classify expressions attached to component
        for expression in expressions:
            # skip nonprinting indicators like annotation
            indicator = expression.indicator
            if (not hasattr(indicator, '_get_lilypond_format') and
                not hasattr(indicator, '_get_lilypond_format_bundle')):
                continue
            elif expression.is_annotation:
                continue
            # skip comments and commands unless attached directly to us
            elif expression.scope is None and \
                hasattr(expression.indicator, '_format_leaf_children') and \
                not getattr(expression.indicator, '_format_leaf_children') and\
                expression.component is not component:
                continue
            # store markup
            elif isinstance(expression.indicator, markuptools.Markup):
                if expression.indicator.direction == Up:
                    up_markup.append(expression.indicator)
                elif expression.indicator.direction == Down:
                    down_markup.append(expression.indicator)
                elif expression.indicator.direction in (Center, None):
                    neutral_markup.append(expression.indicator)
            # store scoped expressions
            elif expression.scope is not None:
                if expression._is_formattable_for_component(component):
                    scoped_expressions.append(expression)
            # store nonscoped expressions
            else:
                nonscoped_expressions.append(expression)
        indicators = (
            up_markup,
            down_markup,
            neutral_markup,
            scoped_expressions,
            nonscoped_expressions,
            )
        return indicators

    @staticmethod
    def _populate_context_setting_format_contributions(component, bundle):
        result = []
        from abjad.tools.topleveltools import set_
        from abjad.tools import scoretools
        manager = LilyPondFormatManager
        if isinstance(component, scoretools.Context):
            for name, value in vars(set_(component)).items():
                string = manager.format_lilypond_context_setting_in_with_block(
                    name, value)
                result.append(string)
        else:
            contextualizer = set_(component)
            variables = vars(contextualizer)
            for name, value in variables.items():
                # if we've found a leaf context namespace
                if name.startswith('_'):
                    for x, y in vars(value).items():
                        if not x.startswith('_'):
                            string = \
                                manager.format_lilypond_context_setting_inline(
                                    x, y, name)
                            result.append(string)
                # otherwise we've found a default leaf context setting
                else:
                    # parse default context setting
                    string = manager.format_lilypond_context_setting_inline(
                        name, value)
                    result.append(string)
        result.sort()
        bundle.context_settings.extend(result)

    @staticmethod
    def _populate_grob_override_format_contributions(component, bundle):
        from abjad.tools import scoretools
        from abjad.tools.topleveltools.override import override
        result = []
        is_once = isinstance(component, scoretools.Leaf)
        contributions = override(component)._list_format_contributions(
            'override',
            is_once=is_once,
            )
        for string in result[:]:
            if 'NoteHead' in string and 'pitch' in string:
                contributions.remove(string)
        bundle.grob_overrides.extend(contributions)

    @staticmethod
    def _populate_grob_revert_format_contributions(component, bundle):
        from abjad.tools import scoretools
        from abjad.tools.topleveltools.override import override
        if not isinstance(component, scoretools.Leaf):
            manager = override(component)
            contributions = manager._list_format_contributions('revert')
            bundle.grob_reverts.extend(contributions)

    @staticmethod
    def _populate_indicator_format_contributions(component, bundle):
        manager = LilyPondFormatManager
        (
            up_markup,
            down_markup,
            neutral_markup,
            scoped_expressions,
            nonscoped_expressions,
            ) = LilyPondFormatManager._collect_indicators(component)
        manager._populate_markup_format_contributions(
            component,
            bundle,
            up_markup,
            down_markup,
            neutral_markup,
            )
        manager._populate_scoped_expression_format_contributions(
            component,
            bundle,
            scoped_expressions,
            )
        manager._populate_nonscoped_expression_format_contributions(
            component,
            bundle,
            nonscoped_expressions,
            )

    @staticmethod
    def _populate_markup_format_contributions(
        component,
        bundle,
        up_markup,
        down_markup,
        neutral_markup,
        ):
        from abjad.tools import markuptools
        for markup_list in (up_markup, down_markup, neutral_markup):
            if not markup_list:
                continue
            elif 1 < len(markup_list):
                direction = markup_list[0].direction
                if direction is None:
                    direction = '-'
                markup_list = markup_list[:]
                markup_list.sort(key=lambda x: -x.stack_priority)
                markup_list = [
                    markuptools.Markup.line([_]) for _ in markup_list]
                markup = markuptools.Markup.column(
                    markup_list,
                    direction=direction,
                    )
                format_pieces = markup._get_format_pieces()
                bundle.right.markup.extend(format_pieces)
            else:
                if markup_list[0].direction is None:
                    markup = markuptools.Markup(markup_list[0], direction='-')
                    format_pieces = markup._get_format_pieces()
                    bundle.right.markup.extend(format_pieces)
                else:
                    format_pieces = markup_list[0]._get_format_pieces()
                    bundle.right.markup.extend(format_pieces)

    @staticmethod
    def _populate_nonscoped_expression_format_contributions(
        component,
        bundle,
        nonscoped_expressions,
        ):
        for nonscoped_expression in nonscoped_expressions:
            indicator = nonscoped_expression.indicator
            if hasattr(indicator, '_get_lilypond_format_bundle'):
                indicator_bundle = indicator._get_lilypond_format_bundle(
                    component)
                if indicator_bundle is not None:
                    bundle.update(indicator_bundle)

    @staticmethod
    def _populate_scoped_expression_format_contributions(
        component,
        bundle,
        scoped_expressions,
        ):
        for scoped_expression in scoped_expressions:
            format_pieces = scoped_expression._get_format_pieces()
            if isinstance(format_pieces, type(bundle)):
                bundle.update(format_pieces)
            else:
                format_slot = scoped_expression.indicator._format_slot
                bundle.get(format_slot).indicators.extend(format_pieces)

    @staticmethod
    def _populate_spanner_format_contributions(
        component,
        bundle,
        ):
        pairs = []
        for spanner in component._get_parentage()._get_spanners():
            spanner_bundle = spanner._get_lilypond_format_bundle(component)
            pair = (spanner, spanner_bundle)
            pairs.append(pair)
        pairs.sort(key=lambda x: type(x[0]).__name__)
        for spanner, spanner_bundle in pairs:
            bundle.update(spanner_bundle)

    ### PUBLIC METHODS ###

    @staticmethod
    def bundle_format_contributions(component):
        r'''Gets all format contributions for `component`.

        Returns LilyPond format bundle.
        '''
        from abjad.tools import systemtools
        manager = LilyPondFormatManager
        bundle = systemtools.LilyPondFormatBundle()
        manager._populate_indicator_format_contributions(component, bundle)
        manager._populate_spanner_format_contributions(component, bundle)
        manager._populate_context_setting_format_contributions(
            component, bundle)
        manager._populate_grob_override_format_contributions(component, bundle)
        manager._populate_grob_revert_format_contributions(component, bundle)
        bundle.alphabetize()
        bundle.make_immutable()
        return bundle

    @staticmethod
    def format_lilypond_attribute(attribute):
        r'''Formats LilyPond attribute according to Scheme formatting
        conventions.

        Returns string.
        '''
        assert isinstance(attribute, str), repr(attribute)
        attribute = attribute.replace('__', ".")
        result = attribute.replace('_', '-')
        return result

    @staticmethod
    def format_lilypond_context_setting_in_with_block(name, value):
        r'''Formats LilyPond context setting `name` with `value`
        in LilyPond with-block.

        Returns string.
        '''
        assert isinstance(name, str), repr(name)
        name = name.split('_')
        first = name[0:1]
        rest = name[1:]
        rest = [x.title() for x in rest]
        name = first + rest
        name = ''.join(name)
        value = LilyPondFormatManager.format_lilypond_value(value)
        value_parts = value.split('\n')
        result = r'{!s} = {!s}'.format(name, value_parts[0])
        result = [result]
        for part in value_parts[1:]:
            result.append(LilyPondFormatManager.indent + part)
        return '\n'.join(result)

    @staticmethod
    def format_lilypond_context_setting_inline(name, value, context=None):
        r'''Formats LilyPond context setting `name` with `value` in
        `context`.

        Returns string.
        '''
        name = name.split('_')
        first = name[0:1]
        rest = name[1:]
        rest = [x.title() for x in rest]
        name = first + rest
        name = ''.join(name)
        value = LilyPondFormatManager.format_lilypond_value(value)
        if context is not None:
            context_string = context[1:]
            context_string = context_string.split('_')
            context_string = [x.title() for x in context_string]
            context_string = ''.join(context_string)
            context_string += '.'
        else:
            context_string = ''
        result = r'\set {}{} = {}'
        result = result.format(context_string, name, value)
        return result

    @staticmethod
    def format_lilypond_value(argument):
        r'''Formats LilyPond `argument` according to Scheme formatting
        conventions.

        Returns string.
        '''
        from abjad.tools import schemetools
        if '_get_lilypond_format' in dir(argument) and not isinstance(argument, str):
            pass
        elif argument in (True, False):
            argument = schemetools.Scheme(argument)
        elif argument in (Up, Down, Left, Right, Center):
            argument = schemetools.Scheme(repr(argument).lower())
        elif isinstance(argument, int) or isinstance(argument, float):
            argument = schemetools.Scheme(argument)
        elif argument in LilyPondFormatManager.lilypond_color_constants:
            argument = schemetools.Scheme(argument)
        elif isinstance(argument, str) and '::' in argument:
            argument = schemetools.Scheme(argument)
        elif isinstance(argument, tuple) and len(argument) == 2:
            argument = schemetools.SchemePair(argument[0], argument[1])
        elif isinstance(argument, str) and ' ' not in argument:
            argument = schemetools.Scheme(argument, quoting="'")
        elif isinstance(argument, str) and ' ' in argument:
            argument = schemetools.Scheme(argument)
        else:
            argument = schemetools.Scheme(argument, quoting="'")
        return format(argument, 'lilypond')

    @staticmethod
    def make_lilypond_override_string(
        grob_name,
        grob_attribute,
        grob_value,
        context_name=None,
        is_once=False,
        ):
        r'''Makes Lilypond override string.

        Does not include 'once'.

        Returns string.
        '''
        from abjad.tools import stringtools
        grob_name = stringtools.to_upper_camel_case(grob_name)
        grob_attribute = LilyPondFormatManager.format_lilypond_attribute(
            grob_attribute)
        grob_value = LilyPondFormatManager.format_lilypond_value(grob_value)
        if context_name is not None:
            context_prefix = stringtools.to_upper_camel_case(context_name)
            context_prefix += '.'
        else:
            context_prefix = ''
        if is_once:
            once_prefix = r'\once '
        else:
            once_prefix = ''
        result = r'{}\override {}{}.{} = {}'
        result = result.format(
            once_prefix,
            context_prefix,
            grob_name,
            grob_attribute,
            grob_value,
            )
        return result

    @staticmethod
    def make_lilypond_revert_string(
        grob_name,
        grob_attribute,
        context_name=None,
        ):
        r'''Makes LilyPond revert string.

        Returns string.
        '''
        from abjad.tools import stringtools
        grob_name = stringtools.to_upper_camel_case(grob_name)
        grob_attribute = LilyPondFormatManager.format_lilypond_attribute(
            grob_attribute)
        # change #'bound-details #'left #'text to #'bound-details
        grob_attribute = grob_attribute.split('.')[0]
        context_prefix = ''
        if context_name is not None:
            context_prefix = stringtools.to_upper_camel_case(context_name)
            context_prefix += '.'
        result = r'\revert {}{}.{}'
        result = result.format(context_prefix, grob_name, grob_attribute)
        return result

    @staticmethod
    def make_lilypond_tweak_string(
        grob_attribute,
        grob_value,
        grob_name=None,
        ):
        r'''Makes Lilypond \tweak string.

        Returns string.
        '''
        from abjad.tools import stringtools
        if grob_name is not None:
            grob_name = stringtools.to_upper_camel_case(grob_name)
            grob_string = grob_name + '.'
        else:
            grob_string = ''
        grob_attribute = LilyPondFormatManager.format_lilypond_attribute(
            grob_attribute)
        grob_value = LilyPondFormatManager.format_lilypond_value(grob_value)
        result = r'- \tweak {}{} {}'
        result = result.format(
            grob_string,
            grob_attribute,
            grob_value,
            )
        return result

    @staticmethod
    def report_component_format_contributions(component, verbose=False):
        r'''Reports `component` format contributions.

            >>> staff = Staff("c'4 [ ( d'4 e'4 f'4 ] )")
            >>> override(staff[0]).note_head.color = 'red'

        ::

            >>> manager = systemtools.LilyPondFormatManager
            >>> print(manager.report_component_format_contributions(staff[0]))
            slot 1:
                grob overrides:
                    \once \override NoteHead.color = #red
            slot 3:
            slot 4:
                leaf body:
                    c'4 [ (
            slot 5:
            slot 7:

        Returns string.
        '''
        return component._report_format_contributors()

    @staticmethod
    def report_spanner_format_contributions(spanner):
        r'''Reports spanner format contributions for every leaf
        to which spanner attaches.

            >>> staff = Staff("c8 d e f")
            >>> spanner = spannertools.Beam()
            >>> attach(spanner, staff[:])

        ::

            >>> manager = systemtools.LilyPondFormatManager
            >>> print(manager.report_spanner_format_contributions(spanner))
            c8	systemtools.LilyPondFormatBundle(
                    right=LilyPondFormatBundle.SlotContributions(
                        spanner_starts=['['],
                        ),
                    )
            d8	systemtools.LilyPondFormatBundle()
            e8	systemtools.LilyPondFormatBundle()
            f8	systemtools.LilyPondFormatBundle(
                    right=LilyPondFormatBundle.SlotContributions(
                        spanner_stops=[']'],
                        ),
                    )

        Returns none or return string.
        '''
        result = []
        for leaf in spanner._get_leaves():
            bundle = spanner._get_lilypond_format_bundle(leaf)
            bundle_pieces = format(bundle).split('\n')
            result.append('{!s}\t{}'.format(leaf, bundle_pieces[0]))
            for piece in bundle_pieces[1:]:
                result.append('\t{}'.format(piece))
        result = '\n'.join(result)
        return result



# -*- coding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools.topleveltools import iterate


def fill_measures_with_time_signature_denominator_notes(
    argument, iterctrl=None):
    r'''Fill measures in `argument` with time signature denominator notes:

    ::

        >>> staff = Staff([Measure((3, 4), []), Measure((3, 16), []), Measure((3, 8), [])])
        >>> scoretools.fill_measures_with_time_signature_denominator_notes(staff)

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            {
                \time 3/4
                c'4
                c'4
                c'4
            }
            {
                \time 3/16
                c'16
                c'16
                c'16
            }
            {
                \time 3/8
                c'8
                c'8
                c'8
            }
        }

    Delete existing contents of measures in `argument`.

    Returns none.
    '''
    from abjad.tools import scoretools

    if iterctrl is None:
        iterctrl = lambda measure, i: True
    for i, measure in enumerate(iterate(argument).by_class(scoretools.Measure)):
        if iterctrl(measure, i):
            time_signature = measure.time_signature
            denominator = mathtools.greatest_power_of_two_less_equal(
                time_signature.denominator)
            numerator = time_signature.numerator
            notes = scoretools.Note(0, (1, denominator)) * numerator
            measure[:] = notes



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_schemetools_SchemePair___setattr___01():
    r'''Scehem pairs are immutable.
    '''

    scheme_pair = schemetools.SchemePair('spacing', 4)
    assert pytest.raises(AttributeError, "scheme_pair.foo = 'bar'")



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_rhythmtreetools_RhythmTreeContainer___getitem___01():

    leaf_a = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=3)
    leaf_b = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=2)
    leaf_c = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1)

    container = rhythmtreetools.RhythmTreeContainer(
        preprolated_duration=1, children=[leaf_a, leaf_b, leaf_c])

    assert container[0] is leaf_a
    assert container[1] is leaf_b
    assert container[2] is leaf_c

    pytest.raises(Exception, 'container[3]')

    assert container[-1] is leaf_c
    assert container[-2] is leaf_b
    assert container[-3] is leaf_a

    pytest.raises(Exception, 'container[-4]')



# -*- coding: utf-8 -*-


def label(client=None):
    r'''Makes label agent or label expression.

    ..  container:: example

        Labels logical ties with start offsets:

        ::

            >>> staff = Staff(r"\times 2/3 { c'4 d'4 e'4 ~ } e'4 ef'4")
            >>> label(staff).with_start_offsets(direction=Up)
            >>> override(staff).text_script.staff_padding = 4
            >>> override(staff).tuplet_bracket.staff_padding = 0
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff \with {
                \override TextScript.staff-padding = #4
                \override TupletBracket.staff-padding = #0
            } {
                \times 2/3 {
                    c'4 ^ \markup { 0 }
                    d'4 ^ \markup { 1/6 }
                    e'4 ~ ^ \markup { 1/3 }
                }
                e'4
                ef'4 ^ \markup { 3/4 }
            }

        See the ``LabelAgent`` API entry for many more examples.

    ..  container:: example expression

        Initializes positionally:

        ::

            >>> expression = label()
            >>> expression(staff)
            LabelAgent(client=<Staff{3}>)

        Initializes from keyword:

        ::

            >>> expression = label()
            >>> expression(client=staff)
            LabelAgent(client=<Staff{3}>)

        Makes label expression:

            >>> expression = label()
            >>> expression = expression.with_start_offsets()

        ::

            >>> staff = Staff(r"\times 2/3 { c'4 d'4 e'4 ~ } e'4 ef'4")
            >>> expression(staff)
            >>> override(staff).text_script.staff_padding = 4
            >>> override(staff).tuplet_bracket.staff_padding = 0
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff \with {
                \override TextScript.staff-padding = #4
                \override TupletBracket.staff-padding = #0
            } {
                \times 2/3 {
                    c'4 ^ \markup { 0 }
                    d'4 ^ \markup { 1/6 }
                    e'4 ~ ^ \markup { 1/3 }
                }
                e'4
                ef'4 ^ \markup { 3/4 }
            }

        See the ``LabelAgent`` API entry for many more examples.

    Returns label agent when `client` is not none.

    Returns label expression when `client` is none.
    '''
    from abjad.tools import agenttools
    from abjad.tools import expressiontools
    if client is not None:
        return agenttools.LabelAgent(client=client)
    expression = expressiontools.Expression()
    expression = expression.label()
    return expression



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Voice___setattr___01():
    r'''Slots constrain voice attributes.
    '''

    voice = Voice([])

    assert pytest.raises(AttributeError, "voice.foo = 'bar'")



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Tuplet_from_duration_and_ratio_01():

    duration = Duration(3, 16)

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1],
        avoid_dots=True,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/2 {
            c'8
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 1],
        avoid_dots=True,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/2 {
            c'16
            c'16
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 1, 1],
        avoid_dots=True,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16
            c'16
            c'16
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 1, 1, 1],
        avoid_dots=True,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/2 {
            c'32
            c'32
            c'32
            c'32
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 1, 1, 1, 1],
        avoid_dots=True,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 6/5 {
            c'32
            c'32
            c'32
            c'32
            c'32
        }
        '''
        )


def test_scoretools_Tuplet_from_duration_and_ratio_02():

    duration = Duration(3, 16)

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1],
        avoid_dots=True,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/2 {
            c'8
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 2],
        avoid_dots=True,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16
            c'8
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 2, 2],
        avoid_dots=True,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 6/5 {
            c'32
            c'16
            c'16
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 2, 2, 3],
        avoid_dots=True,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/2 {
            c'64
            c'32
            c'32
            c'32.
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 2, 2, 3, 3],
        avoid_dots=True,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 12/11 {
            c'64
            c'32
            c'32
            c'32.
            c'32.
        }
        '''
        )


def test_scoretools_Tuplet_from_duration_and_ratio_03():
    r'''Interpret negative proportions as rests.
    '''

    duration = Duration(3, 16)

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, -2, -2, 3, 3],
        avoid_dots=True,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 12/11 {
            c'64
            r32
            r32
            c'32.
            c'32.
        }
        '''
        )


def test_scoretools_Tuplet_from_duration_and_ratio_04():
    r'''Reduce proportions relative to each other.
    '''

    duration = Duration(3, 16)

    tuplet_1 = Tuplet.from_duration_and_ratio(
        duration,
        [1, -2, -2, 3, 3],
        avoid_dots=True,
        is_diminution=False,
        )

    tuplet_2 = Tuplet.from_duration_and_ratio(
        duration,
        [2, -4, -4, 6, 6],
        avoid_dots=True,
        is_diminution=False,
        )

    assert format(tuplet_1) == format(tuplet_2)

    tuplet = Tuplet.from_duration_and_ratio(
        Duration(1, 8),
        [27],
        avoid_dots=True,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'8
        }
        '''
        )


def test_scoretools_Tuplet_from_duration_and_ratio_05():

    duration = Duration(3, 16)

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1],
        avoid_dots=False,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'8.
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 1],
        avoid_dots=False,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16.
            c'16.
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 1, 1],
        avoid_dots=False,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16
            c'16
            c'16
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 1, 1, 1],
        avoid_dots=False,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'32.
            c'32.
            c'32.
            c'32.
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 1, 1, 1, 1],
        avoid_dots=False,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 8/5 {
            c'64.
            c'64.
            c'64.
            c'64.
            c'64.
        }
        '''
        )


def test_scoretools_Tuplet_from_duration_and_ratio_06():

    duration = Duration(3, 16)

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1],
        avoid_dots=False,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'8.
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 2],
        avoid_dots=False,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16
            c'8
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 2, 2],
        avoid_dots=False,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 8/5 {
            c'64.
            c'32.
            c'32.
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 2, 2, 3],
        avoid_dots=False,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/2 {
            c'64
            c'32
            c'32
            c'32.
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 2, 2, 3, 3],
        avoid_dots=False,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 12/11 {
            c'64
            c'32
            c'32
            c'32.
            c'32.
        }
        '''
        )


def test_scoretools_Tuplet_from_duration_and_ratio_07():
    r'''Reduce proportions relative to each other.
    '''

    duration = Duration(3, 16)

    tuplet_1 = Tuplet.from_duration_and_ratio(
        duration,
        [1, 2, 2, 3, 3],
        avoid_dots=False,
        is_diminution=False,
        )

    tuplet_2 = Tuplet.from_duration_and_ratio(
        duration,
        [2, 4, 4, 6, 6],
        avoid_dots=False,
        is_diminution=False,
        )

    assert format(tuplet_1) == format(tuplet_2)

    tuplet = Tuplet.from_duration_and_ratio(
        Duration(1, 8),
        [27], avoid_dots=False,
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'8
        }
        '''
        )


def test_scoretools_Tuplet_from_duration_and_ratio_08():

    duration = Duration(3, 16)

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1],
        avoid_dots=True,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/4 {
            c'4
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 1],
        avoid_dots=True,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/4 {
            c'8
            c'8
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 1, 1],
        avoid_dots=True,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16
            c'16
            c'16
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 1, 1, 1],
        avoid_dots=True,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/4 {
            c'16
            c'16
            c'16
            c'16
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 1, 1, 1, 1],
        avoid_dots=True,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/5 {
            c'16
            c'16
            c'16
            c'16
            c'16
        }
        '''
        )


def test_scoretools_Tuplet_from_duration_and_ratio_09():

    duration = Duration(3, 16)

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1],
        avoid_dots=True,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/4 {
            c'4
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 2],
        avoid_dots=True,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16
            c'8
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 2, 2],
        avoid_dots=True,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/5 {
            c'16
            c'8
            c'8
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 2, 2, 3],
        avoid_dots=True,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/4 {
            c'32
            c'16
            c'16
            c'16.
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 2, 2, 3, 3],
        avoid_dots=True,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 6/11 {
            c'32
            c'16
            c'16
            c'16.
            c'16.
        }
        '''
        )


def test_scoretools_Tuplet_from_duration_and_ratio_10():
    r'''Interpret negative proportions as rests.
    '''

    duration = Duration(3, 16)

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, -2, -2, 3, 3],
        avoid_dots=True,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 6/11 {
            c'32
            r16
            r16
            c'16.
            c'16.
        }
        '''
        )


def test_scoretools_Tuplet_from_duration_and_ratio_11():
    r'''Reduce propotions relative to each other.
    '''

    duration = Duration(3, 16)

    tuplet_1 = Tuplet.from_duration_and_ratio(
        duration,
        [1, -2, -2, 3, 3],
        avoid_dots=True,
        is_diminution=True,
        )

    tuplet_2 = Tuplet.from_duration_and_ratio(
        duration,
        [2, -4, -4, 6, 6],
        avoid_dots=True,
        is_diminution=True,
        )

    assert format(tuplet_1) == format(tuplet_2)

    tuplet = Tuplet.from_duration_and_ratio(Duration(1, 8), [27])

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'8
        }
        '''
        )


def test_scoretools_Tuplet_from_duration_and_ratio_12():

    duration = Duration(3, 16)

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1],
        avoid_dots=False,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'8.
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 1],
        avoid_dots=False,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16.
            c'16.
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 1, 1],
        avoid_dots=False,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16
            c'16
            c'16
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 1, 1, 1],
        avoid_dots=False,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'32.
            c'32.
            c'32.
            c'32.
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 1, 1, 1, 1],
        avoid_dots=False,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \times 4/5 {
            c'32.
            c'32.
            c'32.
            c'32.
            c'32.
        }
        '''
        )


def test_scoretools_Tuplet_from_duration_and_ratio_13():

    duration = Duration(3, 16)

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1],
        avoid_dots=False,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'8.
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 2],
        avoid_dots=False,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16
            c'8
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 2, 2],
        avoid_dots=False,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \times 4/5 {
            c'32.
            c'16.
            c'16.
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 2, 2, 3],
        avoid_dots=False,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/4 {
            c'32
            c'16
            c'16
            c'16.
        }
        '''
        )

    tuplet = Tuplet.from_duration_and_ratio(
        duration,
        [1, 2, 2, 3, 3],
        avoid_dots=False,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 6/11 {
            c'32
            c'16
            c'16
            c'16.
            c'16.
        }
        '''
        )


def test_scoretools_Tuplet_from_duration_and_ratio_14():
    r'''Reduce proportions relative to each other.
    '''

    duration = Duration(3, 16)

    tuplet_1 = Tuplet.from_duration_and_ratio(
        duration,
        [1, -2, -2, 3, 3],
        avoid_dots=False,
        is_diminution=True,
        )

    tuplet_2 = Tuplet.from_duration_and_ratio(
        duration,
        [2, -4, -4, 6, 6],
        avoid_dots=False,
        is_diminution=True,
        )

    assert format(tuplet_1) == format(tuplet_2)

    tuplet = Tuplet.from_duration_and_ratio(
        Duration(1, 8),
        [27],
        avoid_dots=False,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'8
        }
        '''
        )


def test_scoretools_Tuplet_from_duration_and_ratio_15():
    r'''Coerce duration.
    '''

    tuplet = Tuplet.from_duration_and_ratio(
        Duration(1, 4),
        [1, -1, 1],
        avoid_dots=True,
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \times 2/3 {
            c'8
            r8
            c'8
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_Container_append_01():
    r'''Append sequential to voice.
    '''

    voice = Voice("c'8 d'8")
    beam = Beam()
    attach(beam, voice[:])
    voice.append(Container("e'8 f'8"))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8 ]
            {
                e'8
                f'8
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container_append_02():
    r'''Append leaf to tuplet.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    beam = Beam()
    attach(beam, tuplet[:])
    tuplet.append(Note(5, (1, 16)))

    assert format(tuplet) == stringtools.normalize(
        r'''
        \times 4/7 {
            c'8 [
            d'8
            e'8 ]
            f'16
        }
        '''
        )

    assert inspect_(tuplet).is_well_formed()


def test_scoretools_Container_append_03():
    r'''Trying to append noncomponent to container raises TypeError.
    '''

    voice = Voice("c'8 d'8 e'8")
    beam = Beam()
    attach(beam, voice[:])

    assert pytest.raises(Exception, "voice.append('foo')")
    assert pytest.raises(Exception, "voice.append(99)")
    assert pytest.raises(Exception, "voice.append([])")
    assert pytest.raises(Exception, "voice.append([Note(0, (1, 8))])")


def test_scoretools_Container_append_04():
    r'''Append spanned leaf from donor container to recipient container.
    '''

    voice = Voice("c'8 d'8 e'8")
    beam = Beam()
    attach(beam, voice[:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8
            e'8 ]
        }
        '''
        )

    u = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, u[:])

    assert format(u) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8
            e'8
            f'8 ]
        }
        '''
        )

    voice.append(u[-1])

    "Container voice is now ..."

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8
            e'8 ]
            f'8
        }
        '''
        )

    assert inspect_(voice).is_well_formed()

    "Container u is now ..."

    assert format(u) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8
            e'8 ]
        }
        '''
        )

    assert inspect_(u).is_well_formed()


def test_scoretools_Container_append_05():
    r'''Append spanned leaf from donor container to recipient container.
    Donor and recipient containers are the same.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8
            e'8
            f'8 ]
        }
        '''
        )

    voice.append(voice[1])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            e'8
            f'8 ]
            d'8
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container_append_06():
    r'''Can not insert grace container into container.
    '''

    staff = Staff("c' d' e'")
    grace_container = scoretools.GraceContainer("f'16 g'")

    assert pytest.raises(Exception, 'staff.append(grace_container)')



#! /usr/bin/env python
# -*- coding: utf-8 -*-

if __name__ == '__main__':
    from abjad import show
    from abjad.demos import bartok
    lilypond_file = bartok.make_bartok_score()
    show(lilypond_file)



# -*- coding: utf-8 -*-
from abjad import *


def test_instrumenttools_notes_and_chords_are_in_range_01():

    staff = Staff("c'8 r8 <d' fs'>8 r8")
    violin = instrumenttools.Violin()
    attach(violin, staff)

    assert instrumenttools.notes_and_chords_are_in_range(staff)


def test_instrumenttools_notes_and_chords_are_in_range_02():

    staff = Staff("c'8 r8 <d fs>8 r8")
    violin = instrumenttools.Violin()
    attach(violin, staff)

    assert not instrumenttools.notes_and_chords_are_in_range(staff)



# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadObject
from abjad.tools import sequencetools
from abjad.tools import pitchtools


class Mode(AbjadObject):
    '''A diatonic mode.

    Can be extended for nondiatonic mode.

    Modes with different ascending and descending forms not yet implemented.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_named_interval_segment',
        '_mode_name',
        )

    ### INITIALIZER ###

    def __init__(self, mode_name='dorian'):
        if isinstance(mode_name, str):
            mode_name = mode_name
        elif isinstance(mode_name, Mode):
            mode_name = mode_name.mode_name
        else:
            message = 'must be mode or mode name: {!r}.'
            message = message.format(mode_name)
            raise TypeError(message)
        mdi_segment = self._initialize_with_mode_name(mode_name)
        self._named_interval_segment = mdi_segment
        self._mode_name = mode_name

    ### SPECIAL METHODS ###

    def __eq__(self, arg):
        r'''Is true when `arg` is a mode with mode name equal to that of this
        mode. Otherwise false.

        Returns true or false.
        '''
        if not isinstance(arg, type(self)):
            return False
        return self.mode_name == arg.mode_name

    def __hash__(self):
        r'''Hashes mode.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(Mode, self).__hash__()

    def __len__(self):
        r'''Length of mode.

        Returns nonnegative integer.
        '''
        return len(self.named_interval_segment)

    def __ne__(self, arg):
        r'''Is true when `arg` does not equal mode. Otherwise false.

        Returns true or false.
        '''
        return not self == arg

    def __str__(self):
        r'''String representation of mode.

        Returns string.
        '''
        return self.mode_name

    ### PRIVATE METHODS ###

    def _initialize_with_mode_name(self, mode_name):
        mdi_segment = []
        m2 = pitchtools.NamedInterval('minor', 2)
        M2 = pitchtools.NamedInterval('major', 2)
        A2 = pitchtools.NamedInterval('augmented', 2)
        dorian = [M2, m2, M2, M2, M2, m2, M2]
        if mode_name == 'dorian':
            mdi_segment.extend(sequencetools.Sequence(dorian).rotate(n=0))
        elif mode_name == 'phrygian':
            mdi_segment.extend(sequencetools.Sequence(dorian).rotate(n=-1))
        elif mode_name == 'lydian':
            mdi_segment.extend(sequencetools.Sequence(dorian).rotate(n=-2))
        elif mode_name == 'mixolydian':
            mdi_segment.extend(sequencetools.Sequence(dorian).rotate(n=-3))
        elif mode_name in ('aeolian', 'minor', 'natural minor'):
            mdi_segment.extend(sequencetools.Sequence(dorian).rotate(n=-4))
        elif mode_name == 'locrian':
            mdi_segment.extend(sequencetools.Sequence(dorian).rotate(n=-5))
        elif mode_name in ('ionian', 'major'):
            mdi_segment.extend(sequencetools.Sequence(dorian).rotate(n=-6))
        elif mode_name == 'melodic minor':
            mdi_segment.extend([M2, m2, M2, M2, M2, M2, m2])
        elif mode_name == 'harmonic minor':
            mdi_segment.extend([M2, m2, M2, M2, m2, A2, m2])
        else:
            message = 'unknown mode name: {!r}.'
            message = message.format(mode_name)
            raise ValueError(message)
        return pitchtools.IntervalSegment(
            items=mdi_segment,
            item_class=pitchtools.NamedInterval,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def mode_name(self):
        r'''Mode name.

        Returns string.
        '''
        return self._mode_name

    @property
    def named_interval_segment(self):
        r'''Named interval segment of mode.

        Returns named interval segment.
        '''
        return self._named_interval_segment



# -*- coding: utf-8 -*-
from abjad import *
import pytest


def test_spannertools_Spanner__is_my_first_leaf_01():
    r'''Spanner attached to flat container.
    '''

    class MockSpanner(spannertools.Spanner):

        def __init__(self, components=None):
            spannertools.Spanner.__init__(self, components)

    container = Container("c'8 cs'8 d'8 ef'8")
    spanner = MockSpanner()
    attach(spanner, container[:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8
            cs'8
            d'8
            ef'8
        }
        '''
        )

    assert spanner._is_my_first_leaf(container[0])
    for leaf in container[1:]:
        assert not spanner._is_my_first_leaf(leaf)
    assert spanner._is_my_last_leaf(container[-1])
    for leaf in container[:-1]:
        assert not spanner._is_my_last_leaf(leaf)
    for leaf in container:
        assert not spanner._is_my_only_leaf(leaf)


def test_spannertools_Spanner__is_my_first_leaf_02():
    r'''Spanner attached to container with nested contents.
    '''

    class MockSpanner(spannertools.Spanner):

        def __init__(self, components=None):
            spannertools.Spanner.__init__(self, components)

    container = Container(
        r'''
        c'8
        cs'8
        {
            d'8
            ef'8
        }
        e'8
        f'8
        '''
        )

    leaves = select(container).by_leaf()
    spanner = MockSpanner()
    attach(spanner, leaves[:4])

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8
            cs'8
            {
                d'8
                ef'8
            }
            e'8
            f'8
        }
        '''
        )

    assert spanner._is_my_first_leaf(container[0])
    assert spanner._is_my_last_leaf(container[2][1])



# -*- coding: utf-8 -*-
import abc
import bisect
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools import sequencetools
from abjad.tools import spannertools
from abjad.tools.abctools import AbjadObject
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import mutate


class QTarget(AbjadObject):
    r'''Abstract base class from which concrete ``QTarget`` subclasses inherit.

    ``QTarget`` is created by a concrete ``QSchema`` instance, and represents
    the mold into which the timepoints contained by a ``QSequence`` instance
    will be poured, as structured by that ``QSchema`` instance.

    Not composer-safe.

    Used internally by the ``Quantizer``.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_items',
        )

    ### INITIALIZATION ###

    def __init__(self, items=None):
        items = items or []
        #assert len(items)
        assert all(isinstance(x, self.item_class) for x in items)
        self._items = tuple(sorted(items, key=lambda x: x.offset_in_ms))

    ### SPECIAL METHODS ###

    def __call__(
        self,
        q_event_sequence,
        grace_handler=None,
        heuristic=None,
        job_handler=None,
        attack_point_optimizer=None,
        attach_tempos=True
        ):
        r'''Calls q-target.
        '''

        from abjad.tools import quantizationtools

        assert isinstance(q_event_sequence, quantizationtools.QEventSequence)

        if grace_handler is None:
            grace_handler = quantizationtools.ConcatenatingGraceHandler()
        assert isinstance(grace_handler, quantizationtools.GraceHandler)

        if heuristic is None:
            heuristic = quantizationtools.DistanceHeuristic()
        assert isinstance(heuristic, quantizationtools.Heuristic)

        if job_handler is None:
            job_handler = quantizationtools.SerialJobHandler()
        assert isinstance(job_handler, quantizationtools.JobHandler)

        if attack_point_optimizer is None:
            attack_point_optimizer = \
                quantizationtools.NaiveAttackPointOptimizer()
        assert isinstance(
            attack_point_optimizer, quantizationtools.AttackPointOptimizer)

        # if next-to-last QEvent is silent, pop the TerminalQEvent,
        # in order to prevent rest-tuplets
        q_events = q_event_sequence
        if isinstance(q_event_sequence[-2], quantizationtools.SilentQEvent):
            q_events = q_event_sequence[:-1]

        # parcel QEvents out to each beat
        beats = self.beats
        offsets = sorted([beat.offset_in_ms for beat in beats])
        for q_event in q_events:
            index = bisect.bisect(offsets, q_event.offset) - 1
            beat = beats[index]
            beat.q_events.append(q_event)

        # generate QuantizationJobs and process with the JobHandler
        jobs = [beat(i) for i, beat in enumerate(beats)]
        jobs = [job for job in jobs if job]
        jobs = job_handler(jobs)
        for job in jobs:
            beats[job.job_id]._q_grids = job.q_grids

        #for i, beat in enumerate(beats):
        #    print i, len(beat.q_grids)
        #    for q_event in beat.q_events:
        #        print '\t{}'.format(q_event.offset)

        # select the best QGrid for each beat, according to the Heuristic
        beats = heuristic(beats)

        # shift QEvents attached to each QGrid's "next downbeat"
        # over to the next QGrid's first leaf - the real downbeat
        self._shift_downbeat_q_events_to_next_q_grid()

        #  TODO: handle a final QGrid with QEvents attached to its
        #        next_downbeat.
        #  TODO: remove a final QGrid with no QEvents

        # convert the QGrid representation into notation,
        # handling grace-note behavior with the GraceHandler
        return self._notate(
            attach_tempos=attach_tempos,
            attack_point_optimizer=attack_point_optimizer,
            grace_handler=grace_handler,
            )

    ### PRIVATE METHODS ###

    @abc.abstractmethod
    def _notate(
        self,
        grace_handler=None,
        attack_point_optimizer=None,
        attach_tempos=True,
        ):
        pass

    def _notate_leaves(
        self,
        grace_handler=None,
        voice=None,
        ):
        for leaf in iterate(voice).by_leaf():
            if leaf._has_indicator(indicatortools.Annotation):
                annotation = leaf._get_indicator(indicatortools.Annotation)
                pitches, grace_container = grace_handler(annotation.value)
                if not pitches:
                    new_leaf = scoretools.Rest(leaf)
                elif 1 < len(pitches):
                    new_leaf = scoretools.Chord(leaf)
                    new_leaf.written_pitches = pitches
                else:
                    new_leaf = scoretools.Note(leaf)
                    new_leaf.written_pitch = pitches[0]
                if grace_container:
                    attach(grace_container, new_leaf)
                tie = spannertools.Tie()
                if tie._attachment_test(new_leaf):
                    attach(tie, new_leaf)
                mutate(leaf).replace(new_leaf)
            else:
                previous_leaf = leaf._get_leaf(-1)
                if isinstance(previous_leaf, scoretools.Rest):
                    new_leaf = type(previous_leaf)(
                        leaf.written_duration,
                        )
                elif isinstance(previous_leaf, scoretools.Note):
                    new_leaf = type(previous_leaf)(
                        previous_leaf.written_pitch,
                        leaf.written_duration,
                        )
                else:
                    new_leaf = type(previous_leaf)(
                        previous_leaf.written_pitch,
                        leaf.written_duration,
                        )
                mutate(leaf).replace(new_leaf)
                tie = inspect_(previous_leaf).get_spanner(spannertools.Tie)
                if tie is not None:
                    tie._append(new_leaf)
            if leaf._has_indicator(indicatortools.Tempo):
                tempo = leaf._get_indicator(indicatortools.Tempo)
                detach(indicatortools.Tempo, leaf)
                attach(tempo, new_leaf)

    def _shift_downbeat_q_events_to_next_q_grid(self):
        beats = self.beats
        for one, two in sequencetools.Sequence(beats).nwise():
            one_q_events = one.q_grid.next_downbeat.q_event_proxies
            two_q_events = two.q_grid.leaves[0].q_event_proxies
            while one_q_events:
                two_q_events.append(one_q_events.pop())

    ### PUBLIC PROPERTIES ###

    @abc.abstractproperty
    def beats(self):
        r'''Beats of q-target.
        '''
        raise NotImplementedError

    @property
    def duration_in_ms(self):
        r'''Duration of q-target in milliseconds.

        Returns duration.
        '''
        last_item = self._items[-1]
        return last_item.offset_in_ms + last_item.duration_in_ms

    @abc.abstractproperty
    def item_class(self):
        r'''Item class of q-target.
        '''
        raise NotImplementedError

    @property
    def items(self):
        r'''Items of q-target.
        '''
        return self._items



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_make_dynamic_spanner_below_with_nib_at_right_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    spanner = spannertools.make_dynamic_spanner_below_with_nib_at_right('mp')
    attach(spanner, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \override TextSpanner.bound-details.left.text = \markup {
                \dynamic
                    {
                        mp
                    }
                }
            \override TextSpanner.bound-details.right-broken.text = ##f
            \override TextSpanner.bound-details.right.text = \markup {
                \draw-line
                    #'(0 . 1)
                }
            \override TextSpanner.dash-fraction = #1
            \override TextSpanner.direction = #down
            c'8 \startTextSpan
            d'8
            e'8
            f'8 \stopTextSpan
            \revert TextSpanner.bound-details
            \revert TextSpanner.dash-fraction
            \revert TextSpanner.direction
        }
        ''',
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSegment_from_selection_01():
    r'''Works with notes.
    '''

    note = Note(13, (1, 4))
    assert pitchtools.PitchClassSegment.from_selection(
        note,
        item_class=pitchtools.NumberedPitchClass,
        ) == (pitchtools.NumberedPitchClass(1), )


def test_pitchtools_PitchClassSegment_from_selection_02():
    r'''Works with multiple-note chords.
    '''

    chord = Chord([13, 14, 15], (1, 4))
    assert pitchtools.PitchClassSegment.from_selection(
        chord,
        item_class=pitchtools.NumberedPitchClass,
        ) == (
        pitchtools.NumberedPitchClass(1),
        pitchtools.NumberedPitchClass(2),
        pitchtools.NumberedPitchClass(3))


def test_pitchtools_PitchClassSegment_from_selection_03():
    r'''Works with one-note chords.
    '''

    chord = Chord([13], (1, 4))
    assert pitchtools.PitchClassSegment.from_selection(
        chord,
        item_class=pitchtools.NumberedPitchClass,
        ) == (pitchtools.NumberedPitchClass(1), )


def test_pitchtools_PitchClassSegment_from_selection_04():
    r'''Works with empty chords.
    '''

    chord = Chord([], (1, 4))
    assert pitchtools.PitchClassSegment.from_selection(
        chord,
        item_class=pitchtools.NumberedPitchClass,
        ) == ()



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_VerticalMoment_previous_vertical_moment_01():

    score = Score(r'''
        \new Staff {
            \times 4/3 {
                d''8
                c''8
                b'8
            }
        }
        \new PianoStaff <<
            \new Staff {
                a'4
                g'4
            }
        \new Staff {
            \clef "bass"
            f'8
            e'8
            d'8
            c'8
        }
        >>
        ''')

    selector = select().by_leaf(flatten=True)
    leaves = selector(score)
    last_leaf = leaves[-1]
    vertical_moment = inspect_(last_leaf).get_vertical_moment()
    assert vertical_moment.offset == Offset(3, 8)

    vertical_moment = vertical_moment.previous_vertical_moment
    assert vertical_moment.offset == Offset(1, 3)

    vertical_moment = vertical_moment.previous_vertical_moment
    assert vertical_moment.offset == Offset(1, 4)

    vertical_moment = vertical_moment.previous_vertical_moment
    assert vertical_moment.offset == Offset(1, 6)

    vertical_moment = vertical_moment.previous_vertical_moment
    assert vertical_moment.offset == Offset(1, 8)

    vertical_moment = vertical_moment.previous_vertical_moment
    assert vertical_moment.offset == Offset(0)


# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Container___setattr___01():
    r'''Slots constrain container attributes.
    '''

    container = Container([])

    assert pytest.raises(AttributeError, "container.foo = 'bar'")



# -*- coding: utf-8 -*-


def select_all(inverted=None):
    r'''Makes pattern that matches all indices.

    ..  container:: example

        Selects all divisions for tie creation:

        ::

            >>> pattern = patterntools.select_all()

        ::

            >>> print(format(pattern))
            patterntools.Pattern(
                indices=[0],
                period=1,
                )

        ::

            >>> maker = rhythmmakertools.NoteRhythmMaker(
            ...     tie_specifier=rhythmmakertools.TieSpecifier(
            ...         tie_across_divisions=pattern,
            ...         use_messiaen_style_ties=True,
            ...         ),
            ...     )
            >>> divisions = [(7, 16), (3, 8), (7, 16), (3, 8)]
            >>> music = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     music,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    c'4. \repeatTie
                }
                {
                    \time 7/16
                    c'4.. \repeatTie
                }
                {
                    \time 3/8
                    c'4. \repeatTie
                }
            }

    Returns pattern.
    '''
    from abjad.tools import patterntools
    return patterntools.Pattern(
        indices=[0],
        inverted=inverted,
        period=1,
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_schemetools_SchemeColor___init___01():
    r'''Initialize scheme color with string.
    '''

    note = Note("c'4")
    override(note).note_head.color = schemetools.SchemeColor('ForestGreen')

    assert format(note) == stringtools.normalize(
        r'''
        \once \override NoteHead.color = #(x11-color 'ForestGreen)
        c'4
        '''
        )


def test_schemetools_SchemeColor___init___02():
    r'''Normal (non-X11) color names specify with a string.
    '''

    note = Note("c'4")
    override(note).note_head.color = 'grey'

    assert format(note) == stringtools.normalize(
        r'''
        \once \override NoteHead.color = #grey
        c'4
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Leaf_duration_compare_01():
    r'''Written durations can be evaluated for equality with durations.
    '''

    note = Note("c'4")
    assert note.written_duration == Duration(1, 4)


def test_scoretools_Leaf_duration_compare_02():
    r'''Written durations can be evaluated for equality with integers.
    '''

    note = Note(0, 1)
    assert note.written_duration == 1


def test_scoretools_Leaf_duration_compare_03():
    r'''Written durations can NOT be evaluated for equality with tuples.
    '''

    note = Note("c'4")
    assert note.written_duration == Duration(1, 4)
    assert note.written_duration != (1, 4)
    assert note.written_duration != 'foo'



# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_TonalAnalysisAgent_analyze_chords_01():
    r'''The three inversions of a C major triad.
    '''

    chord = Chord([0, 4, 7], (1, 4))
    chord_class = tonalanalysistools.RootedChordClass('c', 'major', 'triad', 'root')
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_chords() == [chord_class]

    chord = Chord([4, 7, 12], (1, 4))
    chord_class = tonalanalysistools.RootedChordClass('c', 'major', 'triad', 1)
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_chords() == [chord_class]

    chord = Chord([7, 12, 16], (1, 4))
    chord_class = tonalanalysistools.RootedChordClass('c', 'major', 'triad', 2)
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_chords() == [chord_class]


def test_tonalanalysistools_TonalAnalysisAgent_analyze_chords_02():
    r'''The three inversions of an a minor triad.
    '''

    chord = Chord([9, 12, 16], (1, 4))
    chord_class = tonalanalysistools.RootedChordClass('a', 'minor', 'triad', 'root')
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_chords() == [chord_class]

    chord = Chord([12, 16, 21], (1, 4))
    chord_class = tonalanalysistools.RootedChordClass('a', 'minor', 'triad', 1)
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_chords() == [chord_class]

    chord = Chord([16, 21, 24], (1, 4))
    chord_class = tonalanalysistools.RootedChordClass('a', 'minor', 'triad', 2)
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_chords() == [chord_class]


def test_tonalanalysistools_TonalAnalysisAgent_analyze_chords_03():
    r'''The four inversions of a C dominant seventh chord.
    '''

    chord = Chord([0, 4, 7, 10], (1, 4))
    chord_class = tonalanalysistools.RootedChordClass('c', 'dominant', 7, 'root')
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_chords() == [chord_class]

    chord = Chord([4, 7, 10, 12], (1, 4))
    chord_class = tonalanalysistools.RootedChordClass('c', 'dominant', 7, 1)
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_chords() == [chord_class]

    chord = Chord([7, 10, 12, 16], (1, 4))
    chord_class = tonalanalysistools.RootedChordClass('c', 'dominant', 7, 2)
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_chords() == [chord_class]

    chord = Chord([10, 12, 16, 19], (1, 4))
    chord_class = tonalanalysistools.RootedChordClass('c', 'dominant', 7, 3)
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_chords() == [chord_class]


def test_tonalanalysistools_TonalAnalysisAgent_analyze_chords_04():
    r'''The five inversions of a C dominant ninth chord.
    '''

    chord = Chord([0, 4, 7, 10, 14], (1, 4))
    chord_class = tonalanalysistools.RootedChordClass('c', 'dominant', 9, 'root')
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_chords() == [chord_class]

    chord = Chord([4, 7, 10, 12, 14], (1, 4))
    chord_class = tonalanalysistools.RootedChordClass('c', 'dominant', 9, 1)
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_chords() == [chord_class]

    chord = Chord([7, 10, 12, 14, 16], (1, 4))
    chord_class = tonalanalysistools.RootedChordClass('c', 'dominant', 9, 2)
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_chords() == [chord_class]

    chord = Chord([10, 12, 14, 16, 19], (1, 4))
    chord_class = tonalanalysistools.RootedChordClass('c', 'dominant', 9, 3)
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_chords() == [chord_class]

    chord = Chord([2, 10, 12, 16, 19], (1, 4))
    chord_class = tonalanalysistools.RootedChordClass('c', 'dominant', 9, 4)
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_chords() == [chord_class]


def test_tonalanalysistools_TonalAnalysisAgent_analyze_chords_05():
    r'''Returns none when chord does not analyze.
    '''

    chord = Chord('<c cs d>4')
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_chords() == [None]



# -*- coding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools.abctools import AbjadValueObject
from abjad.tools.topleveltools import select


class SliceSelectorCallback(AbjadValueObject):
    r'''Slice selector callback.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Callbacks'

    __slots__ = (
        '_apply_to_each',
        '_start',
        '_stop',
        )

    ### INITIALIZER ###

    def __init__(self, start=None, stop=None, apply_to_each=True):
        assert isinstance(start, (int, type(None)))
        assert isinstance(stop, (int, type(None)))
        self._start = start
        self._stop = stop
        self._apply_to_each = bool(apply_to_each)

    ### SPECIAL METHODS ###

    def __call__(self, argument, rotation=None):
        r'''Iterates tuple `argument`.

        ..  container:: example

            For examples:

            ::

                >>> string = r"c'4 \times 2/3 { d'8 r8 e'8 } r16 f'16 g'8 a'4"
                >>> staff = Staff(string)
                >>> show(staff) # doctest: +SKIP

        ..  container:: example

            **Example 1.**

                >>> selector = selectortools.Selector()
                >>> selector = selector.get_slice(start=-4)
                >>> selections = selector(staff)

            ::

                >>> for selection in selections:
                ...     selection
                ...
                Selection([Rest('r16'), Note("f'16"), Note("g'8"), Note("a'4")])

            Returns tuple of selections.

        Returns tuple of selections or tuple of selections with offset.
        '''
        assert isinstance(argument, tuple), repr(argument)
        prototype = (scoretools.Container, selectiontools.Selection)
        result = []
        if self.apply_to_each:
            for subexpr in argument:
                try:
                    subresult = self._get_item(subexpr)
                    if not isinstance(subresult, prototype):
                        subresult = select(subresult)
                    if isinstance(subresult, selectiontools.Selection):
                        if subresult:
                            result.append(subresult)
                    else:
                        result.append(subresult)
                except IndexError:
                    pass
        else:
            try:
                subresult = self._get_item(argument)
                subresult = select(subresult)
                if isinstance(subresult, selectiontools.Selection):
                    if subresult:
                        result.extend(subresult)
                else:
                    result.extend(subresult)
            except IndexError:
                pass
        return tuple(result)

    ### PRIVATE METHODS ###

    def _get_item(self, argument):
        slice_ = slice(self.start, self.stop)
        result = argument.__getitem__(slice_)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def apply_to_each(self):
        r'''Is true if slice selector callback will be applied against the
        contents of each selection, rather than against the sequence of
        selections itself.

        Otherwise false.

        Returns true or false.
        '''
        return self._apply_to_each

    @property
    def start(self):
        r'''Gets slice selector callback start.

        Returns integer.
        '''
        return self._start

    @property
    def stop(self):
        r'''Gets slice selector callback stop.

        Returns integer.
        '''
        return self._stop



# -*- coding: utf-8 -*-
from __future__ import print_function
import collections
import glob
import hashlib
import importlib
import inspect
import os
import posixpath
import platform
import re
import shutil
import subprocess
import sys
import traceback
from abjad.tools.topleveltools import new
from abjad.tools import abctools
from abjad.tools import stringtools
from abjad.tools import systemtools
from docutils import nodes
from docutils.frontend import OptionParser
from docutils.parsers.rst import Parser
from docutils.parsers.rst import directives
from docutils.utils import new_document
from sphinx import addnodes
from sphinx.util import FilenameUniqDict
from sphinx.util.console import bold, red, brown
from sphinx.util.osutil import copyfile, ensuredir
from xml.dom import minidom


class SphinxDocumentHandler(abctools.AbjadObject):

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Document Handlers'

    __slots__ = (
        '_errored',
        )

    _topleveltools_pattern = re.compile(
        r'''
        \b(
            graph |
            show |
            play |
            topleveltools\.graph |
            topleveltools\.show |
            topleveltools\.play
        )\b
        ''',
        re.VERBOSE,
        )

    _image_target_pattern = re.compile('.+-page(\d+)\..+')

    _basic_template = stringtools.normalize(u'''
        <a href="{source_path}" title="{title}" class="{cls}">
            <img src="{target_path}" alt="{alt}"/>
        </a>
        ''')

    _thumbnail_template = stringtools.normalize(u'''
        <a data-lightbox="{group}" href="{target_path}" title="{title}" data-title="{title}" class="{cls}">
            <img src="{thumbnail_path}" alt="{alt}"/>
        </a>
        ''')

    _table_row_open_template = r'''<div class="table-row">'''

    _table_row_close_template = r'''</div>'''

    ### INITIALIZER ###

    def __init__(self):
        self._errored = False

    ### SPHINX EXTENSION SETUP ###

    @staticmethod
    def setup_sphinx_extension(app):
        from abjad.tools import abjadbooktools
        app.add_config_value('abjadbook_ignored_documents', (), 'env')
        app.add_config_value('abjadbook_console_module_names', (), 'env')
        app.add_directive('abjad', abjadbooktools.AbjadDirective)
        app.add_directive('doctest', abjadbooktools.DoctestDirective)
        app.add_directive('import', abjadbooktools.ImportDirective)
        app.add_directive('reveal', abjadbooktools.RevealDirective)
        app.add_directive('shell', abjadbooktools.ShellDirective)
        app.add_directive('thumbnail', abjadbooktools.ThumbnailDirective)
        app.add_javascript('abjad.js')
        app.add_javascript('copybutton.js')
        app.add_javascript('ga.js')
        app.add_stylesheet('abjad.css')
        app.add_node(
            abjadbooktools.abjad_import_block,
            html=[SphinxDocumentHandler.visit_abjad_import_block, None],
            latex=[SphinxDocumentHandler.visit_abjad_import_block, None],
            )
        app.add_node(
            abjadbooktools.abjad_input_block,
            html=[SphinxDocumentHandler.visit_abjad_input_block, None],
            latex=[SphinxDocumentHandler.visit_abjad_input_block, None],
            )
        app.add_node(
            abjadbooktools.abjad_output_block,
            html=[SphinxDocumentHandler.visit_abjad_output_block_html, None],
            latex=[SphinxDocumentHandler.visit_abjad_output_block_latex, None],
            )
        app.add_node(
            abjadbooktools.abjad_reveal_block,
            html=[SphinxDocumentHandler.visit_abjad_reveal_block, None],
            latex=[SphinxDocumentHandler.visit_abjad_reveal_block, None],
            )
        app.add_node(
            abjadbooktools.abjad_thumbnail_block,
            html=[SphinxDocumentHandler.visit_abjad_thumbnail_block_html, None],
            latex=[SphinxDocumentHandler.visit_abjad_thumbnail_block_latex, None],
            )
        app.connect('build-finished', SphinxDocumentHandler.on_build_finished)
        app.connect('builder-inited', SphinxDocumentHandler.on_builder_inited)
        app.connect('doctree-read', SphinxDocumentHandler.on_doctree_read)
        app.connect('env-updated', SphinxDocumentHandler.on_env_updated)
        app.connect(
            'autodoc-process-docstring',
            SphinxDocumentHandler.on_autodoc_process_docstring,
            )

    ### ON BUILDER INITIALIZED ###

    @staticmethod
    def on_autodoc_process_docstring(app, what, name, obj, options, lines):
        string = stringtools.normalize('\n'.join(lines))
        if string:
            lines[:] = string.split('\n')

    @staticmethod
    def on_builder_inited(app):
        app.builder.thumbnails = FilenameUniqDict()
        app.builder.imagedir = '_images'
        stylesheets_directory = os.path.join(
            app.builder.srcdir,
            '_stylesheets',
            )
        image_directory = os.path.join(
            app.builder.outdir,
            app.builder.imagedir,
            'abjadbook',
            )
        if not os.path.exists(image_directory):
            os.makedirs(image_directory)
        if not os.path.exists(stylesheets_directory):
            os.makedirs(stylesheets_directory)
        default_stylesheet = os.path.join(stylesheets_directory, 'default.ly')
        if not os.path.exists(default_stylesheet):
            with open(default_stylesheet, 'w') as file_pointer:
                file_pointer.write('')
        for file_name in os.listdir(stylesheets_directory):
            if os.path.splitext(file_name)[-1] not in ('.ly', '.ily'):
                continue
            source_file_path = os.path.join(
                stylesheets_directory,
                file_name,
                )
            shutil.copy(source_file_path, image_directory)

    ### ON BUILD FINISHED ###

    @staticmethod
    def cleanup_graphviz_svg(app):
        def process_svg(filename, delete_attributes):
            try:
                with open(filename, 'r') as file_pointer:
                    contents = file_pointer.read()
                document = minidom.parseString(contents)
                svg_element = document.getElementsByTagName('svg')[0]
                view_box = svg_element.getAttribute('viewBox')
                view_box = [float(_) for _ in view_box.split()]
                if delete_attributes:
                    if svg_element.attributes.get('height', None):
                        del(svg_element.attributes['height'])
                    if svg_element.attributes.get('width', None):
                        del(svg_element.attributes['width'])
                else:
                    height = '{}pt'.format(int(view_box[-1] * 0.6))
                    width = '{}pt'.format(int(view_box[-2] * 0.6))
                    svg_element.setAttribute('height', height)
                    svg_element.setAttribute('width', width)
                svg_element.setAttribute('preserveAspectRatio', 'xMinYMin')
                with open(filename, 'w') as file_pointer:
                    document.writexml(file_pointer)
            except KeyError:
                traceback.print_exc()
        image_directory = os.path.join(
            app.builder.outdir,
            app.builder.imagedir,
            )
        with systemtools.TemporaryDirectoryChange(image_directory):
            svg_paths = glob.glob('graphviz*.svg')
            for filename in app.builder.status_iterator(
                svg_paths,
                'cleaning-up svg files (A)...',
                brown,
                len(svg_paths),
                ):
                process_svg(filename, delete_attributes=True)
        image_directory = os.path.join(image_directory, 'abjadbook')
        with systemtools.TemporaryDirectoryChange(image_directory):
            svg_paths = glob.glob('graphviz*.svg')
            for filename in app.builder.status_iterator(
                svg_paths,
                'cleaning-up svg files (B)...',
                brown,
                len(svg_paths),
                ):
                process_svg(filename, delete_attributes=False)

    @staticmethod
    def on_build_finished(app, exc):
        try:
            SphinxDocumentHandler.render_thumbnails(app)
        except KeyboardInterrupt:
            sys.exit(1)
        except:
            traceback.print_exc()
        try:
            SphinxDocumentHandler.cleanup_graphviz_svg(app)
            pass
        except KeyboardInterrupt:
            sys.exit(1)
        except:
            traceback.print_exc()

    @staticmethod
    def render_thumbnails(app):
        image_directory = os.path.join(
            app.builder.outdir,
            app.builder.imagedir,
            )
        thumbnail_paths = app.builder.thumbnails
        for path in app.builder.status_iterator(
            thumbnail_paths,
            'rendering gallery thumbnails...',
            brown,
            len(thumbnail_paths),
            ):
            image_name = os.path.basename(path)
            _, suffix = os.path.splitext(image_name)
            if suffix == '.svg':
                continue
            if 'abjadbook' in path:
                image_name = os.path.join('abjadbook', image_name)
            prefix, suffix = os.path.splitext(image_name)
            thumbnail_name = '{}-thumbnail{}'.format(prefix, suffix)
            image_path = os.path.join(image_directory, image_name)
            thumbnail_path = os.path.join(image_directory, thumbnail_name)
            if os.path.exists(thumbnail_path):
                continue
            resize_command = 'convert {} -filter Lanczos -resize 696x {}'
            resize_command = resize_command.format(image_path, thumbnail_path)
            process = subprocess.Popen(
                resize_command,
                shell=True,
                stderr=subprocess.STDOUT,
                stdout=subprocess.PIPE,
                )
            stdout, stderr = process.communicate()
            return_code = process.returncode
            if return_code:
                message = 'Failed to render {}.'
                message = message.format(thumbnail_name)
                app.builder.warn(message)
                app.builder.warn(resize_command)
                if stdout:
                    if sys.version_info[0] == 3:
                        stdout = stdout.decode('utf-8')
                    app.builder.warn(stdout)

    ### ON ENVIRONMENT UPDATED ###

    @staticmethod
    def install_lightbox_static_files(app):
        source_static_path = os.path.join(app.builder.srcdir, '_static')
        target_static_path = os.path.join(app.builder.outdir, '_static')
        source_lightbox_path = os.path.join(source_static_path, 'lightbox2')
        target_lightbox_path = os.path.join(target_static_path, 'lightbox2')
        relative_file_paths = []
        for root, _, file_names in os.walk(source_lightbox_path):
            for file_name in file_names:
                absolute_file_path = os.path.join(root, file_name)
                relative_file_path = os.path.relpath(
                    absolute_file_path,
                    source_static_path,
                    )
                relative_file_paths.append(relative_file_path)
        if os.path.exists(target_lightbox_path):
            shutil.rmtree(target_lightbox_path)
        for relative_file_path in app.builder.status_iterator(
            relative_file_paths,
            'installing lightbox files... ',
            brown,
            len(relative_file_paths),
            ):
            source_path = os.path.join(source_static_path, relative_file_path)
            target_path = os.path.join(target_static_path, relative_file_path)
            target_directory = os.path.dirname(target_path)
            if not os.path.exists(target_directory):
                ensuredir(target_directory)
            copyfile(source_path, target_path)
            if relative_file_path.endswith('.js'):
                app.add_javascript(relative_file_path)
            elif relative_file_path.endswith('.css'):
                app.add_stylesheet(relative_file_path)

    @staticmethod
    def on_env_updated(app, env):
        try:
            SphinxDocumentHandler.install_lightbox_static_files(app)
        except KeyboardInterrupt:
            sys.exit(1)
        except:
            traceback.print_exc()

    ### ON READ ###

    def collect_abjad_input_blocks(self, document):
        def is_valid_node(node):
            prototype = (
                abjadbooktools.abjad_import_block,
                abjadbooktools.abjad_input_block,
                abjadbooktools.abjad_reveal_block,
                )
            return isinstance(node, prototype)
        from abjad.tools import abjadbooktools
        code_blocks = collections.OrderedDict()
        labels = {}
        for block in document.traverse(is_valid_node):
            if isinstance(block, abjadbooktools.abjad_import_block):
                code_block = \
                    abjadbooktools.CodeBlock.from_docutils_abjad_import_block(block)
                if block.get('reveal-label', None):
                    labels[block.get('reveal-label')] = code_block
                code_blocks[block] = code_block
            elif isinstance(block, abjadbooktools.abjad_input_block):
                code_block = \
                    abjadbooktools.CodeBlock.from_docutils_abjad_input_block(block)
                if block.get('reveal-label', None):
                    labels[block.get('reveal-label')] = code_block
                code_blocks[block] = code_block
            else:
                code_block = labels.get(block['reveal-label'], None)
                if code_block is not None:
                    code_block_specifier = code_block.code_block_specifier
                    if code_block_specifier is None:
                        code_block_specifier = abjadbooktools.CodeBlockSpecifier()
                    code_block_specifier = new(
                        code_block_specifier,
                        hide=False,
                        )
                    code_block = new(
                        code_block,
                        code_block_specifier=code_block_specifier,
                        starting_line_number=block.line,
                        )
                    code_blocks[block] = code_block
        return code_blocks

    def collect_python_literal_blocks(self, document, renderable_only=True):
        def is_valid_node(node):
            prototype = (
                nodes.literal_block,
                nodes.doctest_block,
                )
            return isinstance(node, prototype)
        from abjad.tools import abjadbooktools
        should_process = False
        code_blocks = collections.OrderedDict()
        for block in document.traverse(is_valid_node):
            lines = block[0].splitlines()
            if not lines[0].startswith('>>>'):
                continue
            for line in lines:
                if self._topleveltools_pattern.search(line) is not None:
                    should_process = True
            code_block = \
                abjadbooktools.CodeBlock.from_docutils_literal_block(block)
            code_blocks[block] = code_block
        if renderable_only and not should_process:
            code_blocks.clear()
        return code_blocks

    def get_default_stylesheet(self):
        return 'default.ly'

    @staticmethod
    def interpret_code_blocks(app, document):
        import abjad
        from abjad.tools import abjadbooktools
        if SphinxDocumentHandler.should_ignore_document(app, document):
            print()
            message = '    [abjad-book] ignoring'
            print(message)
            return
        try:
            handler = SphinxDocumentHandler()
            abjad_blocks = handler.collect_abjad_input_blocks(document)
            namespace = {}
            namespace.update(abjad.__dict__)
            for module_name in getattr(app.config, 'abjadbook_console_module_names', ()):
                module = importlib.import_module(module_name)
                namespace.update(module.__dict__)
            abjad_console = abjadbooktools.AbjadBookConsole(
                document_handler=handler,
                locals=namespace.copy(),
                )
            literal_blocks = handler.collect_python_literal_blocks(document)
            literal_console = abjadbooktools.AbjadBookConsole(
                document_handler=handler,
                locals=namespace.copy(),
                )
            if abjad_blocks or literal_blocks:
                print()
                handler.interpret_input_blocks(document, abjad_blocks, abjad_console)
                handler.interpret_input_blocks(document, literal_blocks, literal_console)
                handler.rebuild_document(document, abjad_blocks)
                handler.rebuild_document(document, literal_blocks)
            abjad_console.restore_topleveltools_dict()
            literal_console.restore_topleveltools_dict()
        except abjadbooktools.AbjadBookError as e:
            print()
            print(e.args[0])
        except Exception:
            print()
            traceback.print_exc()

    def interpret_input_blocks(
        self,
        document,
        input_blocks,
        console,
        ):
        code_blocks = tuple(input_blocks.values())
        if not code_blocks:
            return
        progress_indicator = systemtools.ProgressIndicator(
            message='    [abjad-book] interpreting',
            total=len(code_blocks),
            verbose=True,
            )
        with progress_indicator:
            for code_block in code_blocks:
                code_block.interpret(console)
                progress_indicator.advance()

    @staticmethod
    def on_doctree_read(app, document):
        SphinxDocumentHandler.style_document(app, document)
        SphinxDocumentHandler.interpret_code_blocks(app, document)

    @staticmethod
    def parse_rst(rst_string):
        from abjad.tools import abjadbooktools
        parser = Parser()
        directives.register_directive(
            'abjad', abjadbooktools.AbjadDirective,
            )
        directives.register_directive(
            'import', abjadbooktools.ImportDirective,
            )
        directives.register_directive(
            'reveal', abjadbooktools.RevealDirective,
            )
        directives.register_directive('shell', abjadbooktools.ShellDirective)
        settings = OptionParser(components=(Parser,)).get_default_values()
        document = new_document('test', settings)
        parser.parse(rst_string, document)
        document = parser.document
        return document

    def rebuild_document(self, document, blocks):
        for old_node, code_block in reversed(tuple(blocks.items())):
            new_nodes = code_block.as_docutils()
            if (
                len(new_nodes) == 1 and
                stringtools.normalize(old_node.pformat()) ==
                stringtools.normalize(new_nodes[0].pformat())
                ):
                continue
            old_node.parent.replace(old_node, new_nodes)

    def register_error(self):
        self._errored = True

    @staticmethod
    def should_ignore_document(app, document):
        if not app.config.abjadbook_ignored_documents:
            return False
        source = document['source']
        for pattern in app.config.abjadbook_ignored_documents:
            if isinstance(pattern, str):
                if pattern in source:
                    return True
            else:
                if pattern.match(source) is not None:
                    return True
        return False

    @staticmethod
    def style_document(app, document):
        def get_unique_parts(parts):
            unique_parts = [parts[0]]
            for part in parts[1:]:
                if part != unique_parts[-1]:
                    unique_parts.append(part)
                else:
                    break
            return unique_parts
        classes_to_attributes = {}
        for desc_node in document.traverse(addnodes.desc):
            if desc_node.get('domain') != 'py':
                continue
            signature_node = desc_node.traverse(addnodes.desc_signature)[0]
            module_name = signature_node.get('module')
            object_name = signature_node.get('fullname')
            object_type = desc_node.get('objtype')
            module = importlib.import_module(module_name)
            if object_type in ('function', 'class'):
                addname_node = signature_node.traverse(addnodes.desc_addname)[0]
                text = addname_node[0].astext()
                parts = [x for x in text.split('.') if x]
                parts = get_unique_parts(parts)
                if parts[0] in ('abjad', 'experimental', 'ide'):
                    parts = parts[-1:]
                if parts:
                    text = '{}.'.format('.'.join(parts))
                else:
                    text = ''
                addname_node[0] = nodes.Text(text)
            if object_type == 'class':
                cls = getattr(module, object_name, None)
                if cls is None:
                    continue
                if cls not in classes_to_attributes:
                    classes_to_attributes[cls] = {}
                    attributes = inspect.classify_class_attrs(cls)
                    for attribute in attributes:
                        classes_to_attributes[cls][attribute.name] = attribute
                if inspect.isabstract(cls):
                    labelnode = addnodes.only(expr='html')
                    labelnode.append(nodes.emphasis(
                        'abstract ',
                        'abstract ',
                        classes=['property'],
                        ))
                    signature_node.insert(0, labelnode)
            elif object_type in ('method', 'attribute', 'staticmethod', 'classmethod'):
                cls_name, attr_name = object_name.split('.')
                cls = getattr(module, cls_name, None)
                if cls is None:
                    continue
                attr = getattr(cls, attr_name)
                inspected_attr = classes_to_attributes[cls][attr_name]
                label_node = addnodes.only(expr='html')
                defining_class = inspected_attr.defining_class
                if defining_class != cls:
                    addname_node = signature_node.traverse(
                        addnodes.desc_addname)[0]
                    if defining_class.__module__.startswith('abjad'):
                        reftarget = defining_class.__module__
                    else:
                        reftarget = '{}.{}'.format(
                            defining_class.__module__,
                            defining_class.__name__,
                            )
                    xref_node = addnodes.pending_xref(
                        '',
                        refdomain='py',
                        refexplicit=True,
                        reftype='class',
                        reftarget=reftarget,
                        )
                    xref_node.append(nodes.literal(
                        '',
                        '{}'.format(defining_class.__name__),
                        classes=['descclassname'],
                        ))
                    html_only_class_name_node = addnodes.only(expr='html')
                    html_only_class_name_node.append(nodes.Text('('))
                    html_only_class_name_node.append(xref_node)
                    html_only_class_name_node.append(nodes.Text(').'))
                    latex_only_class_name_node = addnodes.only(expr='latex')
                    latex_only_class_name_node.append(nodes.Text(
                        '({}).'.format(defining_class.__name__),
                        ))
                    addname_node.clear()
                    addname_node.append(html_only_class_name_node)
                    addname_node.append(latex_only_class_name_node)
                if getattr(attr, '__isabstractmethod__', False):
                    label_node.append(nodes.emphasis(
                        'abstract ',
                        'abstract ',
                        classes=['property'],
                        ))
                if hasattr(attr, 'im_self') and attr.im_self is not None:
                    signature_node.pop(0)
                    label_node.append(nodes.emphasis(
                        'classmethod ',
                        'classmethod ',
                        classes=['property'],
                        ))
                signature_node.insert(0, label_node)

    def unregister_error(self):
        self._errored = False

    ### ON WRITE ###

    @staticmethod
    def find_target_file_paths(
        absolute_directory_path,
        relative_directory_path,
        file_name_pattern,
        pages,
        ):
        #print(file_name_pattern, pages)
        with systemtools.TemporaryDirectoryChange(absolute_directory_path):
            file_name_matches = glob.glob(file_name_pattern)
            file_name_matches = [_ for _ in file_name_matches
                if '-thumbnail' not in _]
        #print('\t', file_name_matches)
        target_file_name_dict = {}
        if len(file_name_matches) == 1 and '-page' not in file_name_matches[0]:
            target_file_name_dict[1] = file_name_matches[0]
        else:
            for file_name_match in file_name_matches:
                re_match = SphinxDocumentHandler._image_target_pattern.match(
                    file_name_match)
                page = int(re_match.groups()[0])
                target_file_name_dict[page] = file_name_match
        target_file_names = []
        found_all_pages = False
        if pages is None:
            target_file_names.extend(target_file_name_dict.values())
            found_all_pages = bool(target_file_names)
        else:
            for page in pages:
                if page in target_file_name_dict:
                    target_file_names.append(target_file_name_dict[page])
            if len(target_file_names) == len(pages):
                found_all_pages = True
        target_file_paths = [posixpath.join(relative_directory_path, _)
            for _ in target_file_names]
        return target_file_paths, found_all_pages

    @staticmethod
    def get_file_base_name(node, image_render_specifier):
        sha1sum = hashlib.sha1()
        sha1sum.update(node[0].encode('utf-8'))
        sha1sum.update(format(image_render_specifier, 'storage').encode('utf-8'))
        if node['renderer'] == 'graphviz':
            layout = str(node.get('layout', '')).encode('utf-8')
            sha1sum.update(layout)
        sha1sum = sha1sum.hexdigest()
        file_base_name = '{}-{}'.format(node['renderer'], sha1sum)
        return file_base_name

    @staticmethod
    def get_image_directory_paths(self):
        absolute_image_directory_path = os.path.join(
            self.builder.outdir,
            self.builder.imagedir,
            'abjadbook',
            )
        relative_image_directory_path = posixpath.join(
            self.builder.imgpath,
            'abjadbook',
            )
        paths = (absolute_image_directory_path, relative_image_directory_path)
        return paths

    @staticmethod
    def get_source_extension(node):
        if node['renderer'] == 'graphviz':
            source_extension = '.dot'
        elif node['renderer'] == 'lilypond':
            source_extension = '.ly'
        return source_extension

    @staticmethod
    def interpret_image_source(
        self,
        node,
        absolute_source_file_path,
        absolute_target_file_path,
        file_format='png',
        ):
        if node['renderer'] == 'graphviz':
            layout = node.get('layout', 'dot')
            render_command = '{} -T{} {} -o {}'.format(
                layout,
                file_format,
                absolute_source_file_path,
                absolute_target_file_path,
                )
        elif node['renderer'] == 'lilypond':
            render_command = 'lilypond --{} -dpixmap-format=pngalpha -dresolution=300 -dno-point-and-click -o {} {}'.format(
                file_format,
                os.path.splitext(absolute_target_file_path)[0],
                absolute_source_file_path,
                )
        process = subprocess.Popen(
            render_command,
            shell=True,
            stderr=subprocess.STDOUT,
            stdout=subprocess.PIPE,
            )
        stdout, stderr = process.communicate()
        return_code = process.returncode
        if return_code:
            self.builder.warn(
                'Failed to render {}.'.format(absolute_target_file_path),
                (self.builder.current_docname, node.line),
                )
            self.builder.warn(render_command)
            if stdout:
                if sys.version_info[0] == 3:
                    stdout = stdout.decode('utf-8')
                self.builder.warn(stdout)
        return return_code

    @staticmethod
    def postprocess_image_target(
        self,
        node,
        absolute_target_file_path,
        no_trim=False,
        no_resize=False,
        ):
        if no_trim and no_resize:
            return 0
        command = 'convert {}'.format(absolute_target_file_path)
        if not no_resize:
            if node['renderer'] == 'lilypond':
                percent = 33
            else:
                percent = 50
            command = '{} -filter Robidoux -resize {!s}%'.format(
                command, percent)
            if platform.system() == 'Windows':
                command = '{}%'.format(command)
        if not no_trim:
            command = '{} -trim'.format(command)
        command = '{} {}'.format(command, absolute_target_file_path)
#        command = SphinxDocumentHandler.dyld_path_setting + '; ' + command
        process = subprocess.Popen(
            command,
            shell=True,
            stderr=subprocess.STDOUT,
            stdout=subprocess.PIPE,
            )
        stdout, stderr = process.communicate()
        return_code = process.returncode
        if return_code:
            self.builder.warn(
                'Failed to render {}.'.format(absolute_target_file_path),
                (self.builder.current_docname, node.line),
                )
            self.builder.warn(command)
            if stdout:
                if sys.version_info[0] == 3:
                    stdout = stdout.decode('utf-8')
                self.builder.warn(stdout)
        return return_code

    @staticmethod
    def render_png_image(self, node):
        from abjad.tools import abjadbooktools
        # Get all file and path parts.
        image_layout_specifier = node.get('image_layout_specifier', None)
        if image_layout_specifier is None:
            image_layout_specifier = abjadbooktools.ImageLayoutSpecifier()
        image_render_specifier = node.get('image_render_specifier', None)
        if image_render_specifier is None:
            image_render_specifier = abjadbooktools.ImageRenderSpecifier()
        pages = image_layout_specifier.pages

        target_extension = '.png'
        file_base_name = SphinxDocumentHandler.get_file_base_name(
            node, image_render_specifier)
        source_extension = SphinxDocumentHandler.get_source_extension(node)
        file_name_pattern = '{}*{}'.format(file_base_name, target_extension)
        absolute_directory_path, relative_directory_path = \
            SphinxDocumentHandler.get_image_directory_paths(self)
        relative_source_file_path = posixpath.join(
            relative_directory_path,
            file_base_name + source_extension,
            )

        # Check for pre-existing target(s).
        target_file_paths, found_all_pages = \
            SphinxDocumentHandler.find_target_file_paths(
                absolute_directory_path, relative_directory_path,
                file_name_pattern, pages)
        if found_all_pages:
            return (relative_source_file_path, target_file_paths)

        # Write and render source to target(s).
        source_file_name = file_base_name + source_extension
        absolute_source_file_path = os.path.join(
            absolute_directory_path, source_file_name)
        target_file_name = file_base_name + target_extension
        absolute_target_file_path = os.path.join(
            absolute_directory_path, target_file_name)
        SphinxDocumentHandler.write_image_source(
            self, node, absolute_source_file_path)
        return_code = SphinxDocumentHandler.interpret_image_source(
            self,
            node,
            absolute_source_file_path,
            absolute_target_file_path,
            file_format='png'
            )
        if return_code:
            return (relative_directory_path, [])

        # Check for target(s).
        target_file_paths, found_all_pages = \
            SphinxDocumentHandler.find_target_file_paths(
                absolute_directory_path, relative_directory_path,
                file_name_pattern, pages)
        if not found_all_pages:
            return (relative_source_file_path, target_file_paths)

        # Trim and resize target(s).
        for target_file_path in target_file_paths:
            _, file_name = os.path.split(target_file_path)
            target_path = os.path.join(absolute_directory_path, file_name)
            return_code = SphinxDocumentHandler.postprocess_image_target(
                self, node, target_path,
                no_resize=image_render_specifier.no_resize,
                no_trim=image_render_specifier.no_trim,
                )

        # Target(s) must exist, so simply return.
        return (relative_source_file_path, target_file_paths)

    @staticmethod
    def render_svg_image(self, node):
        from abjad.tools import abjadbooktools
        # Get all file and path parts.
        image_layout_specifier = node.get('image_layout_specifier', None)
        if image_layout_specifier is None:
            image_layout_specifier = abjadbooktools.ImageLayoutSpecifier()
        image_render_specifier = node.get('image_render_specifier', None)
        if image_render_specifier is None:
            image_render_specifier = abjadbooktools.ImageRenderSpecifier()
        pages = image_layout_specifier.pages

        target_extension = '.svg'
        file_base_name = SphinxDocumentHandler.get_file_base_name(
            node, image_render_specifier)
        source_extension = SphinxDocumentHandler.get_source_extension(node)
        file_name_pattern = '{}*{}'.format(file_base_name, target_extension)
        absolute_directory_path, relative_directory_path = \
            SphinxDocumentHandler.get_image_directory_paths(self)
        relative_source_file_path = posixpath.join(
            relative_directory_path,
            file_base_name + source_extension,
            )

        # Check for pre-existing target(s).
        target_file_paths, found_all_pages = \
            SphinxDocumentHandler.find_target_file_paths(
                absolute_directory_path, relative_directory_path,
                file_name_pattern, pages)
        if found_all_pages:
            return (relative_source_file_path, target_file_paths)

        # Write and render source to target(s).
        source_file_name = file_base_name + source_extension
        absolute_source_file_path = os.path.join(
            absolute_directory_path, source_file_name)
        target_file_name = file_base_name + target_extension
        absolute_target_file_path = os.path.join(
            absolute_directory_path, target_file_name)
        SphinxDocumentHandler.write_image_source(
            self, node, absolute_source_file_path)
        return_code = SphinxDocumentHandler.interpret_image_source(
            self,
            node,
            absolute_source_file_path,
            absolute_target_file_path,
            file_format='svg'
            )
        if return_code:
            return (relative_directory_path, [])

        # Check for target(s).
        target_file_paths, found_all_pages = \
            SphinxDocumentHandler.find_target_file_paths(
                absolute_directory_path, relative_directory_path,
                file_name_pattern, pages)
        if not found_all_pages:
            return (relative_source_file_path, target_file_paths)

        # Target(s) must exist, so simply return.
        return (relative_source_file_path, target_file_paths)

    @staticmethod
    def visit_abjad_import_block(self, node):
        try:
            #print()
            message = bold(red('Found abjad_import_block.'))
            self.builder.warn(message, (self.builder.current_docname, node.line))
            #print(stringtools.normalize(node.pformat()))
        except KeyboardInterrupt:
            sys.exit(1)
        except:
            traceback.print_exc()
        raise nodes.SkipNode

    @staticmethod
    def visit_abjad_input_block(self, node):
        try:
            #print()
            message = bold(red('Found abjad_input_block.'))
            self.builder.warn(message, (self.builder.current_docname, node.line))
            #print(stringtools.normalize(node.pformat()))
        except KeyboardInterrupt:
            sys.exit(1)
        except:
            traceback.print_exc()
        raise nodes.SkipNode

    @staticmethod
    def visit_abjad_output_block_html(self, node):
        from abjad.tools import abjadbooktools
        image_layout_specifier = node.get('image_layout_specifier', None)
        if image_layout_specifier is None:
            image_layout_specifier = abjadbooktools.ImageLayoutSpecifier()
        image_render_specifier = node.get('image_render_specifier', None)
        if image_render_specifier is None:
            image_render_specifier = abjadbooktools.ImageRenderSpecifier()
        if node['renderer'] not in ('graphviz', 'lilypond'):
            raise nodes.SkipNode
        absolute_image_directory_path = os.path.join(
            self.builder.outdir,
            self.builder.imagedir,
            'abjadbook',
            )
        if not os.path.exists(absolute_image_directory_path):
            os.makedirs(absolute_image_directory_path)
        try:
            if node['renderer'] == 'graphviz':
                relative_source_file_path, relative_target_file_paths = \
                    SphinxDocumentHandler.render_svg_image(self, node)
            else:
                relative_source_file_path, relative_target_file_paths = \
                    SphinxDocumentHandler.render_png_image(self, node)
            if image_layout_specifier.with_thumbnail:
                for relative_target_file_path in relative_target_file_paths:
                    self.builder.thumbnails.add_file('', relative_target_file_path)
            css_classes = set(node.get('cls', '').split())
            template = SphinxDocumentHandler._basic_template
            if image_layout_specifier.with_thumbnail:
                css_classes.add('thumbnail')
                template = SphinxDocumentHandler._thumbnail_template
            if image_layout_specifier.with_columns:
                css_classes.add('table-cell')
                stop = len(relative_target_file_paths)
                step = image_layout_specifier.with_columns
                for i in range(0, stop, step):
                    self.body.append(SphinxDocumentHandler._table_row_open_template)
                    paths = relative_target_file_paths[i:i + step]
                    for relative_target_file_path in paths:
                        prefix, suffix = os.path.splitext(relative_target_file_path)
                        if suffix == '.svg':
                            thumbnail_path = relative_target_file_path
                        else:
                            thumbnail_path = '{}-thumbnail{}'.format(prefix, suffix)
                        result = template.format(
                            alt='',
                            cls=' '.join(sorted(css_classes)),
                            group='group-{}'.format(os.path.basename(relative_source_file_path)),
                            source_path=relative_source_file_path,
                            target_path=relative_target_file_path,
                            title='',
                            thumbnail_path=thumbnail_path,
                            )
                        result = stringtools.normalize(result)
                        result = ('    ' + _ for _ in result.splitlines())
                        result = '\n'.join(result)
                        self.body.append(result)
                    self.body.append(SphinxDocumentHandler._table_row_close_template)
            else:
                css_classes.add('abjadbook')
                for relative_target_file_path in relative_target_file_paths:
                    prefix, suffix = os.path.splitext(relative_target_file_path)
                    if suffix == '.svg':
                        thumbnail_path = relative_target_file_path
                    else:
                        thumbnail_path = '{}-thumbnail{}'.format(prefix, suffix)
                    result = template.format(
                        alt='',
                        cls=' '.join(sorted(css_classes)),
                        group='group-{}'.format(os.path.basename(relative_source_file_path)),
                        source_path=relative_source_file_path,
                        target_path=relative_target_file_path,
                        title='',
                        thumbnail_path=thumbnail_path,
                        )
                    result = stringtools.normalize(result)
                    self.body.append(result)
        except KeyboardInterrupt:
            sys.exit(1)
        except:
            traceback.print_exc()
        raise nodes.SkipNode

    @staticmethod
    def visit_abjad_output_block_latex(self, node):
        raise nodes.SkipNode

    @staticmethod
    def visit_abjad_reveal_block(self, node):
        try:
            #print()
            message = bold(red('Found abjad_reveal_block.'))
            self.builder.warn(message, (self.builder.current_docname, node.line))
            #print(stringtools.normalize(node.pformat()))
        except KeyboardInterrupt:
            sys.exit(1)
        except:
            traceback.print_exc()
        raise nodes.SkipNode

    @staticmethod
    def visit_abjad_thumbnail_block_html(self, node):
        try:
            self.builder.thumbnails.add_file('', node['uri'])
            title = node['title']
            classes = ' '.join(node['classes'])
            group = 'group-{}'.format(
                node['group'] if node['group'] else node['uri']
                )
            if node['uri'] in self.builder.images:
                node['uri'] = os.path.join(
                    self.builder.imgpath,
                    self.builder.images[node['uri']],
                    )
            target_path = node['uri']
            prefix, suffix = os.path.splitext(target_path)
            if suffix == '.svg':
                thumbnail_path = target_path
            else:
                thumbnail_path = '{}-thumbnail{}'.format(prefix, suffix)
            output = SphinxDocumentHandler._thumbnail_template.format(
                alt=title,
                group=group,
                target_path=target_path,
                cls=classes,
                thumbnail_path=thumbnail_path,
                title=title,
                )
            self.body.append(output)
        except KeyboardInterrupt:
            sys.exit(1)
        except:
            traceback.print_exc()
        raise nodes.SkipNode

    @staticmethod
    def visit_abjad_thumbnail_block_latex(self, node):
        try:
            print()
            message = bold(red('Found abjad_thumbnail_block.'))
            self.builder.warn(message, (self.builder.current_docname, node.line))
            print(stringtools.normalize(node.pformat()))
        except KeyboardInterrupt:
            sys.exit(1)
        except:
            traceback.print_exc()
        raise nodes.SkipNode

    @staticmethod
    def write_image_source(self, node, absolute_source_file_path):
        with open(absolute_source_file_path, 'w') as file_pointer:
            code = node[0]
            if sys.version_info[0] == 2:
                code = code.encode('utf-8')
            file_pointer.write(code)

    ### PUBLIC PROPERTIES ###

    @property
    def errored(self):
        return self._errored



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_InspectionAgent_get_sounding_pitches_01():

    staff = Staff("<c''' e'''>4 <d''' fs'''>4")
    glockenspiel = instrumenttools.Glockenspiel()
    attach(glockenspiel, staff)
    instrumenttools.transpose_from_sounding_pitch_to_written_pitch(staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \set Staff.instrumentName = \markup { Glockenspiel }
            \set Staff.shortInstrumentName = \markup { Gkspl. }
            <c' e'>4
            <d' fs'>4
        }
        '''
        )

    sounding_pitches = inspect_(staff[0]).get_sounding_pitches()
    assert sounding_pitches == (
        NamedPitch("c'''"),
        NamedPitch("e'''"),
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedInterval___init___01():
    r'''Initialize from positive number.
    '''

    i = pitchtools.NumberedInterval(3)
    assert i.number == 3


def test_pitchtools_NumberedInterval___init___02():
    r'''Initialize from negative number.
    '''

    i = pitchtools.NumberedInterval(-3)
    assert i.number == -3


def test_pitchtools_NumberedInterval___init___03():
    r'''Initialize from other numbered interval.
    '''

    i = pitchtools.NumberedInterval(3)
    j = pitchtools.NumberedInterval(i)
    assert i.number == j.number == 3
    assert i is not j


def test_pitchtools_NumberedInterval___init___04():
    r'''Initialize from named interval.
    '''

    named_interval = pitchtools.NamedInterval('perfect', 4)
    i = pitchtools.NumberedInterval(named_interval)
    assert i.number == 5
